# -*- coding: utf-8 -*-
from abjad.tools import sequencetools
from abjad.tools.abctools import AbjadValueObject


class Tuning(AbjadValueObject):
    r'''Tuning.

    ..  container:: example

        **Example 1.** Violin tuning:

        ::

            >>> indicator = indicatortools.Tuning(
            ...     pitches=('G3', 'D4', 'A4', 'E5'),
            ...     )
            >>> print(format(indicator))
            indicatortools.Tuning(
                pitches=pitchtools.PitchSegment(
                    (
                        pitchtools.NamedPitch('g'),
                        pitchtools.NamedPitch("d'"),
                        pitchtools.NamedPitch("a'"),
                        pitchtools.NamedPitch("e''"),
                        ),
                    item_class=pitchtools.NamedPitch,
                    ),
                )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_pitches',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        pitches=None,
        ):
        from abjad.tools import pitchtools
        self._default_scope = None
        if pitches is not None:
            if isinstance(pitches, type(self)):
                pitches = pitches.pitches
            else:
                pitches = pitchtools.PitchSegment(
                    items=pitches,
                    item_class=pitchtools.NamedPitch,
                    )
        self._pitches = pitches

    ### PUBLIC METHODS ###

    def get_pitch_ranges_by_string_number(self, string_number):
        r'''Gets tuning pitch ranges by string number.

        ..  container:: example

            **Example 1.** Violin tuning:

            ::

                >>> tuning = indicatortools.Tuning(('G3', 'D4', 'A4', 'E5'))
                >>> string_number = indicatortools.StringNumber((2, 3))
                >>> tuning.get_pitch_ranges_by_string_number(string_number)
                (PitchRange(range_string='[A4, A6]'), PitchRange(range_string='[D4, D6]'))

        Returns pitch ranges.
        '''
        from abjad.tools import indicatortools
        string_number = indicatortools.StringNumber(string_number)
        pitch_ranges = self.pitch_ranges
        result = []
        for number in string_number.numbers:
            index = -number
            pitch_range = pitch_ranges[index]
            result.append(pitch_range)
        return tuple(result)

    def get_pitches_by_string_number(self, string_number):
        r'''Gets tuning pitches by string number.

        ..  container:: example

            **Example 1.** Violin tuning:

            ::

                >>> tuning = indicatortools.Tuning(('G3', 'D4', 'A4', 'E5'))
                >>> string_number = indicatortools.StringNumber((2, 3))
                >>> tuning.get_pitches_by_string_number(string_number)
                (NamedPitch("a'"), NamedPitch("d'"))

        Returns named pitches.
        '''
        from abjad.tools import indicatortools
        string_number = indicatortools.StringNumber(string_number)
        result = []
        for number in string_number.numbers:
            index = -number
            pitch = self.pitches[index]
            result.append(pitch)
        return tuple(result)

    def voice_pitch_classes(
        self,
        pitch_classes,
        allow_open_strings=True,
        ):
        r"""Voices `pitch_classes`.

        ..  container:: example

            **Example 1.**

            ::

                >>> tuning = indicatortools.Tuning(('G3', 'D4', 'A4', 'E5'))
                >>> voicings = tuning.voice_pitch_classes(('a',))
                >>> for voicing in voicings:
                ...     voicing
                ...
                (NamedPitch('a'), None, None, None)
                (NamedPitch("a'"), None, None, None)
                (None, NamedPitch("a'"), None, None)
                (None, NamedPitch("a''"), None, None)
                (None, None, NamedPitch("a'"), None)
                (None, None, NamedPitch("a''"), None)
                (None, None, NamedPitch("a'''"), None)
                (None, None, None, NamedPitch("a''"))
                (None, None, None, NamedPitch("a'''"))

            ::

                >>> voicings = tuning.voice_pitch_classes(
                ...     ('a', 'd'),
                ...     allow_open_strings=False,
                ...     )
                >>> for voicing in voicings:
                ...     voicing
                ...
                (NamedPitch('a'), NamedPitch("d''"), None, None)
                (NamedPitch('a'), NamedPitch("d'''"), None, None)
                (NamedPitch('a'), None, NamedPitch("d''"), None)
                (NamedPitch('a'), None, NamedPitch("d'''"), None)
                (NamedPitch('a'), None, None, NamedPitch("d'''"))
                (NamedPitch('a'), None, None, NamedPitch("d''''"))
                (NamedPitch("d'"), NamedPitch("a'"), None, None)
                (NamedPitch("d'"), NamedPitch("a''"), None, None)
                (NamedPitch("d'"), None, NamedPitch("a''"), None)
                (NamedPitch("d'"), None, NamedPitch("a'''"), None)
                (NamedPitch("d'"), None, None, NamedPitch("a''"))
                (NamedPitch("d'"), None, None, NamedPitch("a'''"))
                (NamedPitch("a'"), NamedPitch("d''"), None, None)
                (NamedPitch("a'"), NamedPitch("d'''"), None, None)
                (NamedPitch("a'"), None, NamedPitch("d''"), None)
                (NamedPitch("a'"), None, NamedPitch("d'''"), None)
                (NamedPitch("a'"), None, None, NamedPitch("d'''"))
                (NamedPitch("a'"), None, None, NamedPitch("d''''"))
                (NamedPitch("d''"), NamedPitch("a'"), None, None)
                (NamedPitch("d''"), NamedPitch("a''"), None, None)
                (NamedPitch("d''"), None, NamedPitch("a''"), None)
                (NamedPitch("d''"), None, NamedPitch("a'''"), None)
                (NamedPitch("d''"), None, None, NamedPitch("a''"))
                (NamedPitch("d''"), None, None, NamedPitch("a'''"))
                (None, NamedPitch("a'"), NamedPitch("d''"), None)
                (None, NamedPitch("a'"), NamedPitch("d'''"), None)
                (None, NamedPitch("a'"), None, NamedPitch("d'''"))
                (None, NamedPitch("a'"), None, NamedPitch("d''''"))
                (None, NamedPitch("d''"), NamedPitch("a''"), None)
                (None, NamedPitch("d''"), NamedPitch("a'''"), None)
                (None, NamedPitch("d''"), None, NamedPitch("a''"))
                (None, NamedPitch("d''"), None, NamedPitch("a'''"))
                (None, NamedPitch("a''"), NamedPitch("d''"), None)
                (None, NamedPitch("a''"), NamedPitch("d'''"), None)
                (None, NamedPitch("a''"), None, NamedPitch("d'''"))
                (None, NamedPitch("a''"), None, NamedPitch("d''''"))
                (None, NamedPitch("d'''"), NamedPitch("a''"), None)
                (None, NamedPitch("d'''"), NamedPitch("a'''"), None)
                (None, NamedPitch("d'''"), None, NamedPitch("a''"))
                (None, NamedPitch("d'''"), None, NamedPitch("a'''"))
                (None, None, NamedPitch("d''"), NamedPitch("a''"))
                (None, None, NamedPitch("d''"), NamedPitch("a'''"))
                (None, None, NamedPitch("a''"), NamedPitch("d'''"))
                (None, None, NamedPitch("a''"), NamedPitch("d''''"))
                (None, None, NamedPitch("d'''"), NamedPitch("a''"))
                (None, None, NamedPitch("d'''"), NamedPitch("a'''"))
                (None, None, NamedPitch("a'''"), NamedPitch("d'''"))
                (None, None, NamedPitch("a'''"), NamedPitch("d''''"))

        Returns tuple of sequences.
        """
        from abjad.tools import pitchtools
        pitch_classes = [pitchtools.NamedPitchClass(x) for x in pitch_classes]
        pitch_classes.extend([None] * (len(self.pitches) - len(pitch_classes)))
        enumeration = sequencetools.Enumeration(pitch_classes)
        permutations = enumeration.yield_permutations()
        permutations = set([tuple(_) for _ in permutations])
        pitch_ranges = self.pitch_ranges
        result = []
        for permutation in permutations:
            sequences = []
            for pitch_range, pitch_class in zip(pitch_ranges, permutation):
                if pitch_class is None:
                    sequences.append([None])
                    continue
                pitches = pitch_range.voice_pitch_class(pitch_class)
                if not allow_open_strings:
                    pitches = [pitch for pitch in pitches
                        if pitch != pitch_range.start_pitch
                        ]
                if not pitches:
                    pitches = [None]
                sequences.append(pitches)
            enumeration = sequencetools.Enumeration(sequences)
            subresult = enumeration.yield_outer_product()
            subresult = [tuple(x) for x in subresult]
            result.extend(subresult)
        result.sort()
        return tuple(result)

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of tuning.

        ..  container:: example

            **Example 1.** Violin tuning

            ::

                >>> indicator = indicatortools.Tuning(
                ...     pitches=('G3', 'D4', 'A4', 'E5'),
                ...     )
                >>> indicator.default_scope is None
                True

        Returns none.
        '''
        return self._default_scope

    @property
    def pitch_ranges(self):
        r'''Gets two-octave pitch-ranges for each pitch in this tuning.

        ..  container:: example

            **Example 1.**

            ::

                >>> pitch_ranges = indicator.pitch_ranges
                >>> print(format(pitch_ranges))
                pitchtools.PitchRangeList(
                    [
                        pitchtools.PitchRange(
                            range_string='[G3, G5]',
                            ),
                        pitchtools.PitchRange(
                            range_string='[D4, D6]',
                            ),
                        pitchtools.PitchRange(
                            range_string='[A4, A6]',
                            ),
                        pitchtools.PitchRange(
                            range_string='[E5, E7]',
                            ),
                        ]
                    )

        Returns pitch-range list.
        '''
        from abjad.tools import pitchtools
        result = []
        for pitch in self.pitches:
            pitch_range = pitchtools.PitchRange.from_pitches(pitch, pitch + 24)
            result.append(pitch_range)
        result = pitchtools.PitchRangeList(result)
        return result

    @property
    def pitches(self):
        r'''Gets pitches of tuning.

        ..  container:: example

            **Example 1.**

            ::

                >>> pitches = indicator.pitches
                >>> print(format(pitches))
                pitchtools.PitchSegment(
                    (
                        pitchtools.NamedPitch('g'),
                        pitchtools.NamedPitch("d'"),
                        pitchtools.NamedPitch("a'"),
                        pitchtools.NamedPitch("e''"),
                        ),
                    item_class=pitchtools.NamedPitch,
                    )

        Returns pitch segment.
        '''
        return self._pitches



# -*- coding: utf-8 -*-
import numbers


def all_are_numbers(argument):
    '''Is true when `argument` is an iterable collection of numbers.
    Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.all_are_numbers([1, 2, 3.0, Fraction(13, 8)])
            True

        ::

            >>> mathtools.all_are_numbers(17)
            False

    ..  container:: example

        Is true when `argument` is empty:

        ::

            >>> mathtools.all_are_numbers([])
            True

    Returns true or false.
    '''
    try:
        return all(isinstance(_, numbers.Number) for _ in argument)
    except TypeError:
        return False



# -*- coding: utf-8 -*-
from abjad import Fraction


def fraction_to_proper_fraction(rational):
    '''Changes `rational` to proper fraction.

    ..  container:: example

        ::

            >>> mathtools.fraction_to_proper_fraction(Fraction(116, 8))
            (14, Fraction(1, 2))

    Returns pair.
    '''
    if not isinstance(rational, Fraction):
        raise TypeError
    quotient = int(rational)
    residue = rational - quotient
    return quotient, residue



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchSet_duplicate_pitch_classes_01():

    pitch_set = pitchtools.PitchSet([0, 12, 13, 26])

    assert pitch_set.duplicate_pitch_classes == pitchtools.PitchClassSet([0])


def test_pitchtools_PitchSet_duplicate_pitch_classes_02():

    pitch_set = pitchtools.PitchSet([0, 13, 26])

    assert pitch_set.duplicate_pitch_classes == pitchtools.PitchClassSet([])



# -*- coding: utf-8 -*-
from abjad.tools.mathtools.Infinity import Infinity


class NegativeInfinity(Infinity):
    r'''Negative infinity.

    ..  container:: example

        All numbers compare greater than negative infinity:

        ::

            >>> NegativeInfinity < -9999999
            True

    ..  container:: example

        Negative infinity compares equal to itself:

        ::

            >>> NegativeInfinity == NegativeInfinity
            True

    ..  container:: example

        Negative infinity compares less than infinity:

        ::

            >>> NegativeInfinity < Infinity
            True

    Initializes as a system singleton at start-up.

    Available as a built-in after Abjad start.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self):
        self._value = float('-infinity')



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_InspectionAgent_get_markup_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    slur = Slur()
    attach(slur, staff[:])
    markup_1 = markuptools.Markup('foo')
    attach(markup_1, staff[0])
    markup_2 = markuptools.Markup('bar')
    attach(markup_2, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 (
                - \markup {
                    \column
                        {
                            \line
                                {
                                    foo
                                }
                            \line
                                {
                                    bar
                                }
                        }
                    }
            d'8
            e'8
            f'8 )
        }
        '''
        )

    markup = inspect_(staff[0]).get_markup()
    assert len(markup) == 2
    assert markup_1 in markup
    assert markup_2 in markup


def test_agenttools_InspectionAgent_get_markup_02():

    chord = Chord([-11, 2, 5], (1, 4))
    up_markup = markuptools.Markup('UP', Up)
    attach(up_markup, chord)
    down_markup = markuptools.Markup('DOWN', Down)
    attach(down_markup, chord)
    found_markup = inspect_(chord).get_markup(direction=Down)
    assert found_markup == (down_markup,)


def test_agenttools_InspectionAgent_get_markup_03():

    chord = Chord([-11, 2, 5], (1, 4))
    up_markup = markuptools.Markup('UP', Up)
    attach(up_markup, chord)
    down_markup = markuptools.Markup('DOWN', Down)
    attach(down_markup, chord)
    found_markup = inspect_(chord).get_markup(direction=Up)
    assert found_markup == (up_markup,)



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_OctavationSpanner_adjust_automatically_01():

    container = Container(scoretools.make_notes([24, 26, 27, 29], [(1, 8)]))

    octavation_spanner = spannertools.OctavationSpanner()
    attach(octavation_spanner, container[:])
    octavation_spanner.adjust_automatically(ottava_breakpoint=14)

    assert format(container) == stringtools.normalize(
        r"""
        {
            \ottava #1
            c'''8
            d'''8
            ef'''8
            f'''8
            \ottava #0
        }
        """
        )

    assert inspect_(container).is_well_formed()


def test_spannertools_OctavationSpanner_adjust_automatically_02():

    container = Container([Note(31, (1, 4))])
    assert container[0].written_pitch.diatonic_pitch_number == 18

    octavation_spanner = spannertools.OctavationSpanner()
    attach(octavation_spanner, container[:])
    octavation_spanner.adjust_automatically(
        ottava_breakpoint=15,
        quindecisima_breakpoint=19,
        )

    assert format(container) == stringtools.normalize(
        r"""
        {
            \ottava #1
            g'''4
            \ottava #0
        }
        """
        )

    assert inspect_(container).is_well_formed()



# -*- coding: utf-8 -*-
'''
Abjad-IPython Extension
-----------------------

Integrates audio and visual rendering of Abjad scores in IPython notebooks.

This extension requires `timidity` be in your $PATH. If you do not have
`timidity` installed, it is likely available in your platform's package
manager:

On OSX::

    ~$ brew install timidity

On Debian or Ubuntu:

    ~$ apt-get install timidity

To activate the IPython notebook extension, add the following line in your
notebook:

    %load_ext abjad.ext.ipython

'''


def load_ipython_extension(ipython):
    import abjad
    from abjad.tools import ipythontools
    from abjad.tools import topleveltools
    play = ipythontools.Play()
    show = ipythontools.Show()
    graph = ipythontools.Graph()
    abjad.play = play
    abjad.show = show
    abjad.graph = graph
    topleveltools.play = play
    topleveltools.show = show
    topleveltools.graph = graph
    names = {
        'play': play,
        'show': show,
        'graph': graph,
        }
    ipython.push(names)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSegment___getslice___01():
    r'''Returns new pitch-class segment.
    '''

    pitch_class_tokens = [-2, -1.5, 6, 7, -1.5, 7]
    pitch_class_segment_1 = pitchtools.PitchClassSegment(pitch_class_tokens)
    pitch_class_segment_2 = pitch_class_segment_1[:2]

    assert isinstance(pitch_class_segment_2, pitchtools.PitchClassSegment)
    assert pitch_class_segment_2 == \
        pitchtools.PitchClassSegment(pitch_class_tokens[:2])


def test_pitchtools_PitchClassSegment___getslice___02():
    r'''Returns new pitch-class segment.
    '''

    pitch_class_tokens = [-2, -1.5, 6, 7, -1.5, 7]
    pitch_class_segment_1 = pitchtools.PitchClassSegment(pitch_class_tokens)
    pitch_class_segment_2 = pitch_class_segment_1[-2:]

    assert isinstance(pitch_class_segment_2, pitchtools.PitchClassSegment)
    assert pitch_class_segment_2 == \
        pitchtools.PitchClassSegment(pitch_class_tokens[-2:])



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_Tempo___div___01():

    tempo_1 = Tempo(Duration(1, 4), 60)
    tempo_2 = Tempo(Duration(1, 4), 90)

    assert tempo_2 / tempo_1 == Duration(3, 2)
    assert tempo_1 / tempo_2 == Duration(2, 3)


def test_indicatortools_Tempo___div___02():

    tempo_1 = Tempo(Duration(1, 8), 42)
    tempo_2 = Tempo(Duration(1, 4), 90)

    assert tempo_2 / tempo_1 == Duration(15, 14)
    assert tempo_1 / tempo_2 == Duration(14, 15)


def test_indicatortools_Tempo___div___03():

    tempo_1 = Tempo(textual_indication='Langsam')
    tempo_2 = Tempo(Duration(1, 4), 90)

    statement = "tempo_1 / tempo_2"
    pytest.raises(ImpreciseTempoError, statement)


def test_indicatortools_Tempo___div___04():

    tempo_1 = Tempo(Duration(1, 8), (90, 92))
    tempo_2 = Tempo(Duration(1, 4), 90)

    statement = "tempo_1 / tempo_2"
    pytest.raises(ImpreciseTempoError, statement)



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_InspectionAgent_has_effective_indicator_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    attach('foo', staff[2], scope=Staff)

    assert not inspect_(staff).has_effective_indicator(str)
    assert not inspect_(staff[0]).has_effective_indicator(str)
    assert not inspect_(staff[1]).has_effective_indicator(str)
    assert inspect_(staff[2]).has_effective_indicator(str)
    assert inspect_(staff[3]).has_effective_indicator(str)



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class PageBreak(AbjadValueObject):
    r'''Page break.

    ..  container:: example

        **Example 1.** Default page break:

        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> page_break = indicatortools.PageBreak()
            >>> attach(page_break, staff[-1])
            >>> show(staff) # doctest: +SKIP

        ::

            >>> page_break
            PageBreak()

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                c'4
                d'4
                e'4
                f'4
                \pageBreak
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        )

    _format_slot = 'closing'

    ### INITIALIZER ##

    def __init__(self):
        from abjad.tools import scoretools
        self._default_scope = scoretools.Staff

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return r'\pageBreak'

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of page break.

        ..  container:: example

            **Example 1.** Default page break:

            ::

                
                >>> page_break = indicatortools.PageBreak()
                >>> page_break.default_scope
                <class 'abjad.tools.scoretools.Staff.Staff'>

        Page breaks are staff-scoped by default.

        ..  todo:: Page breaks should be score-scoped.

        Returns staff (but should return score).
        '''
        return self._default_scope



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_TrillSpanner_pitch_01():
    r'''Works with pitch.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    trill = spannertools.TrillSpanner(pitch=NamedPitch(1))
    attach(trill, staff[:2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \pitchedTrill
            c'8 \startTrillSpan cs'
            d'8 \stopTrillSpan
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_TrillSpanner_pitch_02():
    r'''Works with no pitch.
    '''

    staff = Staff("c'8 d'8 e'8 f'8")
    trill = spannertools.TrillSpanner()
    attach(trill, staff[:2])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \startTrillSpan
            d'8 \stopTrillSpan
            e'8
            f'8
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from sphinx.util.compat import Directive
from docutils.parsers.rst import directives


class ThumbnailDirective(Directive):
    r'''A thumbnail directive.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Sphinx Internals'

    final_argument_whitespace = True
    has_content = False
    option_spec = {
        'class': directives.class_option,
        'group': directives.unchanged,
        'title': directives.unchanged,
        }
    optional_arguments = 0
    required_arguments = 1

    ### PUBLIC METHODS ###

    def run(self):
        r'''Executes the directive.
        '''
        from abjad.tools import abjadbooktools
        node = abjadbooktools.abjad_thumbnail_block()
        node['classes'] += self.options.get('class', '')
        node['group'] = self.options.get('group', '')
        node['title'] = self.options.get('title', '')
        node['uri'] = self.arguments[0]
        environment = self.state.document.settings.env
        environment.images.add_file('', node['uri'])
        return [node]



# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__lilypondfile__ScoreBlock_01():

    target = lilypondfiletools.Block(name='score')
    target.items.append(Score())
    parser = LilyPondParser()
    result = parser(format(target))
    assert format(target) == format(result) and target is not result



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchSet___eq___01():
    r'''Pitch set equality works as expected.
    '''

    pitch_set_1 = pitchtools.PitchSet([12, 14, 18, 19])
    pitch_set_2 = pitchtools.PitchSet([12, 14, 18, 19])
    pitch_set_3 = pitchtools.PitchSet([12, 14, 18, 20])

    assert pitch_set_1 == pitch_set_2
    assert pitch_set_1 != pitch_set_3
    assert pitch_set_2 != pitch_set_3
    assert not pitch_set_1 != pitch_set_2



# -*- coding: utf-8 -*-
from abjad.tools.pitchtools.Vector import Vector


class PitchVector(Vector):
    r'''Pitch vector.

    ..  container:: example

        ::

            >>> vector = pitchtools.PitchVector(
            ...     items=[7, 6, -2, -3, -3, 0, 1, 14, 15, 16, 16],
            ...     item_class=pitchtools.NumberedPitch,
            ...     )

        ::

            >>> items = list(vector.items())
            >>> items.sort(key=lambda x: x[0].pitch_number)
            >>> for pitch_class, count in items:
            ...     print(pitch_class, count)
            -3 2
            -2 1
            0 1
            1 1
            6 1
            7 1
            14 1
            15 1
            16 2

    '''

    ### SPECIAL METHODS ###

    def __repr__(self):
        r'''Gets interpreter representation of pitch vector.

        ..  container:: example

            Gets interpreter representation of pitch vector:
            ::

                >>> vector = pitchtools.PitchVector(
                ...     items=[7, 6, -2, -3, -3, 0, 1, 14, 15, 16, 16],
                ...     item_class=pitchtools.NumberedPitch,
                ...     )

            ::

                >>> vector
                PitchVector({-3: 2, -2: 1, 0: 1, 1: 1, 6: 1, 7: 1, 14: 1, 15: 1, 16: 2}, item_class=NumberedPitch)

        ..  container:: example

            Initializes from interpreter representation of pitch vector:

                >>> pitchtools.PitchVector(vector)
                PitchVector({-3: 2, -2: 1, 0: 1, 1: 1, 6: 1, 7: 1, 14: 1, 15: 1, 16: 2}, item_class=NumberedPitch)

        Returns string.
        '''
        superclass = super(PitchVector, self)
        return superclass.__repr__()

    ### PRIVATE PROPERTIES ###

    @property
    def _named_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NamedPitch

    @property
    def _numbered_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.NumberedPitch

    @property
    def _parent_item_class(self):
        from abjad.tools import pitchtools
        return pitchtools.Pitch

    ### PUBLIC METHODS ###

    @classmethod
    def from_selection(
        class_,
        selection,
        item_class=None,
        ):
        r'''Makes pitch vector from `selection`.

        Returns pitch vector.
        '''
        from abjad.tools import pitchtools
        pitch_segment = pitchtools.PitchSegment.from_selection(selection)
        return class_(
            pitch_segment,
            item_class=item_class,
            )



# -*- coding: utf-8 -*-
import platform
import unittest
from abjad.tools import abjadbooktools
from abjad.tools import stringtools


@unittest.skipIf(
    platform.python_implementation() != 'CPython',
    'Only for CPython',
    )
class TestLaTeXDocumentHandler_exception(unittest.TestCase):

    maxDiff = None

    def test_exception_1(self):
        input_file_contents = [
            '<abjad>',
            "'foo' / 19",
            '</abjad>',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            abjadbooktools.AbjadBookError,
            document_handler.__call__,
            )

    def test_exception_2(self):
        input_file_contents = [
            '<abjad>[allow_exceptions=true]',
            "'foo' / 19",
            '</abjad>',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        rebuilt_source = document_handler(return_source=True)
        assert rebuilt_source == stringtools.normalize(
            '''
            <abjad>[allow_exceptions=true]
            'foo' / 19
            </abjad>

            %%% ABJADBOOK START %%%
            \\begin{lstlisting}
            >>> 'foo' / 19
            Traceback (most recent call last):
              File "<stdin>", line 1, in <module>
            TypeError: unsupported operand type(s) for /: 'str' and 'int'
            \\end{lstlisting}
            %%% ABJADBOOK END %%%
            ''',
            )

    def test_exception_3(self):
        input_file_contents = [
            '<abjad>[allow_exceptions=true]',
            "'foo' / 19",
            '</abjad>',
            '',
            '<abjad>',
            "23 + 'baz'",
            '</abjad>',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        self.assertRaises(
            abjadbooktools.AbjadBookError,
            document_handler.__call__,
            )



# -*- coding: utf-8 -*-
from __future__ import print_function
import os
import pytest
import sys
import traceback
import ide
abjad_ide = ide.tools.idetools.AbjadIDE()


if __name__ == '__main__':
    this_file = os.path.abspath(__file__)
    test_directory = os.path.dirname(this_file)
    inner_score_directory = os.path.dirname(test_directory)
    outer_score_directory = os.path.dirname(inner_score_directory)
    composer_scores_directory = os.path.dirname(outer_score_directory)
    # Travis monkey patch
    abjad_ide._configuration._composer_scores_directory_override = \
        composer_scores_directory
    segments_directory = abjad_ide._to_score_directory(this_file, 'segments')
    segment_directories = abjad_ide._list_visible_paths(segments_directory)

    # not parameterized to print keep-alive message to Travis log
    for segment_directory in segment_directories:
        message = 'Checking {} definition file ...'
        message = message.format(abjad_ide._trim_path(segment_directory))
        print(message)
        try:
            abjad_ide.check_definition_file(segment_directory)
        except:
            traceback.print_exc()
            sys.exit(1)

    # not parameterized to print keep-alive message to Travis log
    for segment_directory in segment_directories:
        message = 'Making {} PDF ...'
        message = message.format(abjad_ide._trim_path(segment_directory))
        print(message)
        try:
            abjad_ide.make_pdf(segment_directory)
        except:
            traceback.print_exc()
            sys.exit(1)



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Parentage_is_orphan_01():

    staff = Staff("c'8 d'8 e'8 f'8")

    assert inspect_(staff).get_parentage().is_orphan
    for note in staff:
        assert not inspect_(note).get_parentage().is_orphan



# -*- coding: utf-8 -*-


def is_null_tuple(argument):
    r'''Is true when `argument` is a tuple of length 0. Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.is_null_tuple(())
            True

        ::

            >>> mathtools.is_null_tuple((19, 20, 21))
            False

    Returns true or false.
    '''
    return isinstance(argument, tuple) and not len(argument)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_LilyPondComment___setattr___01():
    r'''Slots constrain comment attributes.
    '''

    comment = indicatortools.LilyPondComment('foo')

    assert pytest.raises(AttributeError, "comment.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import sequencetools
from abjad.tools.spannertools.ComplexBeam import ComplexBeam
from abjad.tools.topleveltools import inspect_


class DuratedComplexBeam(ComplexBeam):
    r'''Durated complex beam.

    ..  container:: example

        Two groups:

        ::

            >>> staff = Staff("c'16 d'16 e'16 f'16 g'16")
            >>> set_(staff).auto_beaming = False
            >>> durations = [Duration(2, 16), Duration(3, 16)]
            >>> beam = spannertools.DuratedComplexBeam(
            ...     durations=durations,
            ...     span_beam_count=1,
            ...     )
            >>> attach(beam, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                autoBeaming = ##f
            } {
                \set stemLeftBeamCount = #0
                \set stemRightBeamCount = #2
                c'16 [
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #1
                d'16
                \set stemLeftBeamCount = #1
                \set stemRightBeamCount = #2
                e'16
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #2
                f'16
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #0
                g'16 ]
            }

    ..  container:: example

        Two different groups:

        ::

            >>> staff = Staff("c'16 d'16 e'16 f'16 g'16")
            >>> set_(staff).auto_beaming = False
            >>> durations = [Duration(3, 16), Duration(2, 16)]
            >>> beam = spannertools.DuratedComplexBeam(
            ...     durations=durations,
            ...     span_beam_count=1,
            ...     )
            >>> attach(beam, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff \with {
                autoBeaming = ##f
            } {
                \set stemLeftBeamCount = #0
                \set stemRightBeamCount = #2
                c'16 [
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #2
                d'16
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #1
                e'16
                \set stemLeftBeamCount = #1
                \set stemRightBeamCount = #2
                f'16
                \set stemLeftBeamCount = #2
                \set stemRightBeamCount = #0
                g'16 ]
            }

    Groups leaves in spanner according to `durations`.

    Spans leaves between groups according to `span_beam_count`.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_durations',
        '_isolated_nib_direction',
        '_nibs_towards_nonbeamable_components',
        '_span_beam_count',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        beam_rests=None,
        direction=None,
        durations=None,
        isolated_nib_direction=False,
        nibs_towards_nonbeamable_components=True,
        overrides=None,
        span_beam_count=1,
        ):
        ComplexBeam.__init__(
            self,
            beam_rests=beam_rests,
            direction=direction,
            isolated_nib_direction=isolated_nib_direction,
            overrides=overrides,
            )
        durations = self._coerce_durations(durations)
        self._durations = durations
        assert isinstance(nibs_towards_nonbeamable_components, bool)
        self._nibs_towards_nonbeamable_components = \
            nibs_towards_nonbeamable_components
        assert isinstance(span_beam_count, (int, type(None)))
        self._span_beam_count = span_beam_count

    ### PRIVATE METHODS ###

    @staticmethod
    def _coerce_durations(durations):
        durations = durations or ()
        assert isinstance(durations, (list, tuple))
        durations = [durationtools.Duration(x) for x in durations]
        durations = tuple(durations)
        return durations

    def _copy_keyword_args(self, new):
        ComplexBeam._copy_keyword_args(self, new)
        if self.durations is not None:
            new._durations = self.durations[:]
        new._span_beam_count = self.span_beam_count

    def _format_before_leaf(self, leaf):
        import abjad
        if (not isinstance(leaf, abjad.Leaf) or
            not self._is_beamable(leaf)):
            return []
        result = []
        if self._is_exterior_leaf(leaf):
            left, right = self._get_left_right_for_exterior_leaf(leaf)
        elif self._is_just_left_of_gap(leaf):
            left = leaf.written_duration.flag_count
            if self.nibs_towards_nonbeamable_components:
                right = self.span_beam_count
            else:
                next_leaf = inspect_(leaf).get_leaf(1)
                if self._is_beamable(
                    next_leaf,
                    beam_rests=self.beam_rests,
                    ):
                    right = self.span_beam_count
                else:
                    right = 0
        elif self._is_just_right_of_gap(leaf):
            if self.nibs_towards_nonbeamable_components:
                left = self.span_beam_count
            else:
                previous_leaf = inspect_(leaf).get_leaf(-1)
                if self._is_beamable(
                    previous_leaf,
                    beam_rests=self.beam_rests,
                    ):
                    left = self.span_beam_count
                else:
                    left = 0
            right = leaf.written_duration.flag_count
        else:
            assert self._is_interior_leaf(leaf)
            left, right = self._get_left_right_for_interior_leaf(leaf)
        if left is not None:
            string = r'\set stemLeftBeamCount = #{}'.format(left)
            result.append(string)
        if right is not None:
            string = r'\set stemRightBeamCount = #{}'.format(right)
            result.append(string)
        return result

    def _fracture_left(self, i):
        self, left, right = ComplexBeam._fracture_left(self, i)
        weights = [
            inspect_(left).get_duration(),
            inspect_(right).get_duration(),
            ]
        assert sum(self.durations) == sum(weights)
        split_durations = sequencetools.Sequence(self.durations)
        split_durations = split_durations.split(
            weights,
            cyclic=False,
            overhang=False,
            )
        left_durations, right_durations = split_durations
        left._durations = left_durations
        right._durations = right_durations
        return self, left, right

    def _fracture_right(self, i):
        self, left, right = ComplexBeam._fracture_right(self, i)
        weights = [
            inspect_(left).get_duration(),
            inspect_(right).get_duration(),
            ]
        assert sum(self.durations) == sum(weights)
        split_durations = sequencetools.Sequence(self.durations)
        split_durations = split_durations.split(
            weights,
            cyclic=False,
            overhang=False,
            )
        left_durations, right_durations = split_durations
        left._durations = left_durations
        right._durations = right_durations
        return self, left, right

    def _get_span_beam_offsets(self):
        offsets = []
        if self.durations:
            offset = durationtools.Offset(self.durations[0])
            offsets.append(offset)
            for duration in self.durations[1:]:
                offset = offsets[-1] + duration
                offsets.append(offset)
            offsets.pop()
        return offsets

    def _is_just_left_of_gap(self, leaf):
        local_start_offset = self._start_offset_in_me(leaf)
        local_stop_offset = self._stop_offset_in_me(leaf)
        span_beam_offsets = self._get_span_beam_offsets()
        #print(leaf, local_start_offset, local_stop_offset, span_beam_offsets)
        if local_stop_offset in span_beam_offsets:
            #if local_start_offset not in span_beam_offsets:
            if True:
                return True
        return False

    def _is_just_right_of_gap(self, leaf):
        local_start_offset = self._start_offset_in_me(leaf)
        local_stop_offset = self._stop_offset_in_me(leaf)
        span_beam_offsets = self._get_span_beam_offsets()
        if local_start_offset in span_beam_offsets:
            #if local_stop_offset not in span_beam_offsets:
            if True:
                return True
        return False

    def _reverse_components(self):
        ComplexBeam._reverse_components(self)
        durations = reversed(self.durations)
        durations = tuple(durations)
        self._durations = durations

    ### PUBLIC PROPERTIES ###

    @property
    def beam_rests(self):
        r'''Is true when beam should include rests and skips.
        Otherwise false.

        ..  container:: example

            Does not beam rests:

            ::

                >>> staff = Staff("c'8 r r d'")
                >>> beam = spannertools.DuratedComplexBeam(beam_rests=False)
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #1
                    c'8 [ ]
                    r8
                    r8
                    \set stemLeftBeamCount = #1
                    \set stemRightBeamCount = #0
                    d'8 [ ]
                }

            This is default behavior.

        ..  container:: example

            Beams rests:

            ::

                >>> staff = Staff("c'8 r r d'")
                >>> beam = spannertools.DuratedComplexBeam(beam_rests=True)
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #1
                    c'8 [
                    r8
                    r8
                    \set stemLeftBeamCount = #1
                    \set stemRightBeamCount = #0
                    d'8 ]
                }

        ..  container:: example

            Beams skips:

            ::

                >>> staff = Staff("c'8 s s d'")
                >>> beam = spannertools.DuratedComplexBeam(beam_rests=False)
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #1
                    c'8 [ ]
                    s8
                    s8
                    \set stemLeftBeamCount = #1
                    \set stemRightBeamCount = #0
                    d'8 [ ]
                }

            This is default behavior.

        ..  container:: example

            Beams skips:

            ::

                >>> staff = Staff("c'8 s s d'")
                >>> beam = spannertools.DuratedComplexBeam(beam_rests=True)
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #1
                    c'8 [
                    s8
                    s8
                    \set stemLeftBeamCount = #1
                    \set stemRightBeamCount = #0
                    d'8 ]
                }

        ..  container:: example

            Beams large skip with skip at end:

            ::

                >>> string = "c'8 s4 d'8 s8"
                >>> staff = Staff(string)
                >>> beam = spannertools.DuratedComplexBeam(beam_rests=True)
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #1
                    c'8 [
                    s4
                    \set stemLeftBeamCount = #1
                    \set stemRightBeamCount = #1
                    d'8
                    s8 ]
                }

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        superclass = super(DuratedComplexBeam, self)
        return superclass.beam_rests

    @property
    def durations(self):
        r'''Gets durations.

        ..  container:: example

            Two groups:

            ::

                >>> staff = Staff("c'16 d'16 e'16 f'16")
                >>> durations = [Duration(1, 8), Duration(1, 8)]
                >>> beam = spannertools.DuratedComplexBeam(
                ...     durations=durations,
                ...     )
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #2
                    c'16 [
                    \set stemLeftBeamCount = #2
                    \set stemRightBeamCount = #1
                    d'16
                    \set stemLeftBeamCount = #1
                    \set stemRightBeamCount = #2
                    e'16
                    \set stemLeftBeamCount = #2
                    \set stemRightBeamCount = #0
                    f'16 ]
                }

            ::

                >>> beam.durations
                (Duration(1, 8), Duration(1, 8))

        ..  container:: example

            Three groups:

            ::

                >>> staff = Staff("c'16 d'16 e'8 f'16 c'16")
                >>> set_(staff).auto_beaming = False
                >>> durations = [Duration(1, 8), Duration(1, 8), Duration(1, 8)]
                >>> beam = spannertools.DuratedComplexBeam(
                ...     durations=durations,
                ...     span_beam_count=1,
                ...     )
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff \with {
                    autoBeaming = ##f
                } {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #2
                    c'16 [
                    \set stemLeftBeamCount = #2
                    \set stemRightBeamCount = #1
                    d'16
                    \set stemLeftBeamCount = #1
                    \set stemRightBeamCount = #1
                    e'8
                    \set stemLeftBeamCount = #1
                    \set stemRightBeamCount = #2
                    f'16
                    \set stemLeftBeamCount = #2
                    \set stemRightBeamCount = #0
                    c'16 ]
                }

            Test ensures that leaf groups format correctly when they contain
            only one leaf.

        Defaults to none.

        Set to durations or none.

        Returns tuple of durations or none.
        '''
        return self._durations

    @property
    def nibs_towards_nonbeamable_components(self):
        r'''Is true when when spanner should render nibs pointing towards
        nonbeamable components included in spanner.
        Otherwise false.

        ..  container:: example

            Does not draw nibs towards nonbeamable components:

            ::

                >>> staff = Staff("c'16 d'16 r4 e'16 f'16")
                >>> durations = [Duration(1, 8), Duration(1, 4), Duration(1, 8)]
                >>> beam = spannertools.DuratedComplexBeam(
                ...     durations=durations,
                ...     nibs_towards_nonbeamable_components=False
                ...     )
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #2
                    c'16 [
                    \set stemLeftBeamCount = #2
                    \set stemRightBeamCount = #0
                    d'16 ]
                    r4
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #2
                    e'16 [
                    \set stemLeftBeamCount = #2
                    \set stemRightBeamCount = #0
                    f'16 ]
                }

        ..  container:: example

            Draws nibs towards nonbeamable components:

            ::

                >>> staff = Staff("c'16 d'16 r4 e'16 f'16")
                >>> durations = [Duration(1, 8), Duration(1, 4), Duration(1, 8)]
                >>> beam = spannertools.DuratedComplexBeam(
                ...     durations=durations,
                ...     nibs_towards_nonbeamable_components=True
                ...     )
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #2
                    c'16 [
                    \set stemLeftBeamCount = #2
                    \set stemRightBeamCount = #1
                    d'16 ]
                    r4
                    \set stemLeftBeamCount = #1
                    \set stemRightBeamCount = #2
                    e'16 [
                    \set stemLeftBeamCount = #2
                    \set stemRightBeamCount = #0
                    f'16 ]
                }

        Defaults to true.

        Set to true or false.

        Returns true or false.
        '''
        return self._nibs_towards_nonbeamable_components

    @property
    def span_beam_count(self):
        r'''Gets span beam count.

        ..  container:: example

            Creates a single span beam between adjacent groups in spanner:

            ::

                >>> staff = Staff("c'32 d'32 e'32 f'32")
                >>> durations = [Duration(1, 16), Duration(1, 16)]
                >>> beam = spannertools.DuratedComplexBeam(
                ...     durations=durations,
                ...     span_beam_count=1,
                ...     )
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #3
                    c'32 [
                    \set stemLeftBeamCount = #3
                    \set stemRightBeamCount = #1
                    d'32
                    \set stemLeftBeamCount = #1
                    \set stemRightBeamCount = #3
                    e'32
                    \set stemLeftBeamCount = #3
                    \set stemRightBeamCount = #0
                    f'32 ]
                }

            ::

                >>> beam.span_beam_count
                1

        ..  container:: example

            Creates a double span beam between adjacent groups in spanner:

            ::

                >>> staff = Staff("c'32 d'32 e'32 f'32")
                >>> durations = [Duration(1, 16), Duration(1, 16)]
                >>> beam = spannertools.DuratedComplexBeam(
                ...     durations=durations,
                ...     span_beam_count=2,
                ...     )
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #3
                    c'32 [
                    \set stemLeftBeamCount = #3
                    \set stemRightBeamCount = #2
                    d'32
                    \set stemLeftBeamCount = #2
                    \set stemRightBeamCount = #3
                    e'32
                    \set stemLeftBeamCount = #3
                    \set stemRightBeamCount = #0
                    f'32 ]
                }

            ::

                >>> beam.span_beam_count
                2

        ..  container:: example

            Creates no span beam between adjacent groups in spanner:

            ::

                >>> staff = Staff("c'32 d'32 e'32 f'32")
                >>> durations = [Duration(1, 16), Duration(1, 16)]
                >>> beam = spannertools.DuratedComplexBeam(
                ...     durations=durations,
                ...     span_beam_count=0,
                ...     )
                >>> attach(beam, staff[:])
                >>> show(staff) # doctest: +SKIP

            ..  doctest::

                >>> f(staff)
                \new Staff {
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #3
                    c'32 [
                    \set stemLeftBeamCount = #3
                    \set stemRightBeamCount = #0
                    d'32
                    \set stemLeftBeamCount = #0
                    \set stemRightBeamCount = #3
                    e'32
                    \set stemLeftBeamCount = #3
                    \set stemRightBeamCount = #0
                    f'32 ]
                }

            ::

                >>> beam.span_beam_count
                0

        Defaults to ``1``.

        Set to nonnegative integer.

        Returns nonnegative integer.
        '''
        return self._span_beam_count



# -*- coding: utf-8 -*-
from abjad import *


def test_templatetools_GroupedRhythmicStavesScoresTemplate___call___01():

    template = templatetools.GroupedRhythmicStavesScoreTemplate(staff_count=4)
    score = template()

    assert format(score) == stringtools.normalize(
        r'''
        \context Score = "Grouped Rhythmic Staves Score" <<
            \context StaffGroup = "Grouped Rhythmic Staves Staff Group" <<
                \context RhythmicStaff = "Staff 1" {
                    \context Voice = "Voice 1" {
                    }
                }
                \context RhythmicStaff = "Staff 2" {
                    \context Voice = "Voice 2" {
                    }
                }
                \context RhythmicStaff = "Staff 3" {
                    \context Voice = "Voice 3" {
                    }
                }
                \context RhythmicStaff = "Staff 4" {
                    \context Voice = "Voice 4" {
                    }
                }
            >>
        >>
        '''
        )


def test_templatetools_GroupedRhythmicStavesScoresTemplate___call___02():

    template = templatetools.GroupedRhythmicStavesScoreTemplate(staff_count=[2, 1, 2])
    score = template()

    assert format(score) == stringtools.normalize(
        r'''
        \context Score = "Grouped Rhythmic Staves Score" <<
            \context StaffGroup = "Grouped Rhythmic Staves Staff Group" <<
                \context RhythmicStaff = "Staff 1" <<
                    \context Voice = "Voice 1-1" {
                    }
                    \context Voice = "Voice 1-2" {
                    }
                >>
                \context RhythmicStaff = "Staff 2" {
                    \context Voice = "Voice 2" {
                    }
                }
                \context RhythmicStaff = "Staff 3" <<
                    \context Voice = "Voice 3-1" {
                    }
                    \context Voice = "Voice 3-2" {
                    }
                >>
            >>
        >>
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_instrumenttools_notes_and_chords_are_on_expected_clefs_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    clef = Clef(name='treble')
    attach(clef, staff)
    violin = instrumenttools.Violin()
    attach(violin, staff)

    assert instrumenttools.notes_and_chords_are_on_expected_clefs(staff)

    staff = Staff("c'8 d'8 e'8 f'8")
    clef = Clef(name='alto')
    attach(clef, staff)
    violin = instrumenttools.Violin()
    attach(violin, staff)

    assert not instrumenttools.notes_and_chords_are_on_expected_clefs(staff)


def test_instrumenttools_notes_and_chords_are_on_expected_clefs_02():

    staff = Staff("c'8 d'8 e'8 f'8")
    clef = Clef(name='treble')
    attach(clef, staff)
    bassoon = instrumenttools.Bassoon()
    attach(bassoon, staff)

    assert not instrumenttools.notes_and_chords_are_on_expected_clefs(staff)

    staff = Staff("c'8 d'8 e'8 f'8")
    clef = Clef(name='alto')
    attach(clef, staff)
    bassoon = instrumenttools.Bassoon()
    attach(bassoon, staff)

    assert not instrumenttools.notes_and_chords_are_on_expected_clefs(staff)


def test_instrumenttools_notes_and_chords_are_on_expected_clefs_03():

    staff = Staff("c'8 d'8 e'8 f'8")
    clef = Clef(name='percussion')
    attach(clef, staff)
    violin = instrumenttools.Violin()
    attach(violin, staff)

    assert instrumenttools.notes_and_chords_are_on_expected_clefs(
        staff, percussion_clef_is_allowed = True)

    assert not instrumenttools.notes_and_chords_are_on_expected_clefs(
        staff, percussion_clef_is_allowed = False)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchRange_list_octave_transpositions_01():
    r'''Works on chords.
    '''

    chord = Chord([0, 2, 4], (1, 4))
    pitch_range = pitchtools.PitchRange.from_pitches(0, 48)
    transpositions = pitch_range.list_octave_transpositions(chord)

    r"""
    [Chord(c' d' e', 4),
    Chord(c'' d'' e'', 4),
    Chord(c''' d''' e''', 4),
    Chord(c'''' d'''' e'''', 4)]
    """

    assert len(transpositions) == 4
    assert format(transpositions[0]) == "<c' d' e'>4"
    assert format(transpositions[1]) ==  "<c'' d'' e''>4"
    assert format(transpositions[2]) == "<c''' d''' e'''>4"
    assert format(transpositions[3]) == "<c'''' d'''' e''''>4"


def test_pitchtools_PitchRange_list_octave_transpositions_02():
    r'''Works on pitch numbers.
    '''

    pitch_numbers = [0, 2, 4]
    pitch_range = pitchtools.PitchRange.from_pitches(0, 48)
    transpositions = pitch_range.list_octave_transpositions(pitch_numbers)

    assert transpositions == [[0, 2, 4], [12, 14, 16], [24, 26, 28], [36, 38, 40]]



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_indicatortools_Articulation___init___01():
    r'''Articulations can be initialized from zero, one or two arguments.
    '''

    articulation = Articulation('^\\marcato')
    assert articulation.name == 'marcato'
    assert articulation.direction == Up
    articulation = Articulation('legato', Down)
    assert articulation.name == 'legato'
    assert articulation.direction == Down


def test_indicatortools_Articulation___init___02():
    r'''Articulations have string and direction.
    '''

    note = Note("c'4")
    articulation = Articulation('staccato')
    attach(articulation, note)
    assert articulation.name == 'staccato'
    assert articulation.direction is None


def test_indicatortools_Articulation___init___03():
    r'''Direction can be set to None.
    '''

    note = Note("c'4")
    articulation = Articulation('staccato', None)
    attach(articulation, note)
    assert articulation.direction is None
    assert str(articulation) == r'-\staccato'


def test_indicatortools_Articulation___init___04():
    r'''Direction can be set to up.
    '''

    note = Note("c'4")
    articulation = Articulation('staccato', Up)
    attach(articulation, note)
    assert articulation.direction == Up
    assert str(articulation) == r'^\staccato'

    articulation = Articulation('staccato', '^')
    assert articulation.direction == Up
    assert str(articulation) == r'^\staccato'


def test_indicatortools_Articulation___init___05():
    r'''Direction can be set to down.
    '''

    note = Note("c'4")
    articulation = Articulation('staccato', Down)
    attach(articulation, note)
    assert articulation.direction == Down
    assert str(articulation) == r'_\staccato'

    articulation = Articulation('staccato', '_')
    assert articulation.direction == Down
    assert str(articulation) == r'_\staccato'


def test_indicatortools_Articulation___init___06():
    r'''Direction can be set to default.
    '''

    note = Note("c'4")
    articulation = Articulation('staccato')
    assert articulation.direction is None
    assert str(articulation) == r'-\staccato'

    articulation = Articulation('staccato', '-')
    assert articulation.direction == Center
    assert str(articulation) == r'-\staccato'



def test_indicatortools_Articulation___init___07():
    r'''Shortcut strings are replaced with full word.
    '''

    note = Note("c'4")
    articulation = Articulation('.')
    attach(articulation, note)
    assert articulation.name == '.'
    assert str(articulation) == r'-\staccato'

    articulation = Articulation('-')
    assert articulation.name == '-'
    assert str(articulation) == r'-\tenuto'

    articulation = Articulation('|')
    assert articulation.name == '|'
    assert str(articulation) == r'-\staccatissimo'



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_Quantizer___call___01():
    milliseconds = [1500, 1500]
    q_events = quantizationtools.QEventSequence.from_millisecond_durations(
        milliseconds)
    quantizer = quantizationtools.Quantizer()
    result = quantizer(q_events)
    staff = scoretools.Staff([result], context_name='RhythmicStaff')
    score = Score([staff])
    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new RhythmicStaff {
                \new Voice {
                    {
                        \tempo 4=60
                        \time 4/4
                        c'4.
                        c'4.
                        r4
                    }
                }
            }
        >>
        '''
        ), format(score)


def test_quantizationtools_Quantizer___call___02():
    milliseconds = [750, 750]
    q_events = quantizationtools.QEventSequence.from_millisecond_durations(
        milliseconds)
    quantizer = quantizationtools.Quantizer()
    optimizer = quantizationtools.MeasurewiseAttackPointOptimizer()
    result = quantizer(
        q_events,
        attack_point_optimizer=optimizer,
        )
    staff = scoretools.Staff([result], context_name='RhythmicStaff')
    score = Score([staff])
    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new RhythmicStaff {
                \new Voice {
                    {
                        \tempo 4=60
                        \time 4/4
                        c'8.
                        c'16 ~
                        c'8
                        r8
                        r2
                    }
                }
            }
        >>
        '''
        ), format(score)


def test_quantizationtools_Quantizer___call___03():

    milliseconds = [1500, -1000, 1000, 1000, -1000, 1000, -1000, 500]
    sequence = quantizationtools.QEventSequence.from_millisecond_durations(milliseconds)
    attack_point_optimizer = quantizationtools.NullAttackPointOptimizer()
    quantizer = quantizationtools.Quantizer()

    result = quantizer(sequence, attack_point_optimizer=attack_point_optimizer)

    assert isinstance(result, Voice)
    assert inspect_(result).get_duration() == 2

    score = Score([Staff([result])])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \new Voice {
                    {
                        \tempo 4=60
                        \time 4/4
                        c'4 ~
                        c'8
                        r8
                        r8
                        c'8 ~
                        c'8
                        c'8 ~
                    }
                    {
                        c'8
                        r8
                        r8
                        c'8 ~
                        c'8
                        r8
                        r8
                        c'8
                    }
                }
            }
        >>
        '''
        ), format(score)


def test_quantizationtools_Quantizer___call___04():

    milliseconds = [250, 1000, 1000, 1000, 750]
    sequence = quantizationtools.QEventSequence.from_millisecond_durations(
        milliseconds)
    attack_point_optimizer = quantizationtools.NullAttackPointOptimizer()
    quantizer = quantizationtools.Quantizer()
    result = quantizer(sequence, attack_point_optimizer=attack_point_optimizer)
    staff = scoretools.Staff([result], context_name='RhythmicStaff')
    score = Score([staff])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new RhythmicStaff {
                \new Voice {
                    {
                        \tempo 4=60
                        \time 4/4
                        c'16
                        c'16 ~
                        c'8 ~
                        c'16
                        c'16 ~
                        c'8 ~
                        c'16
                        c'16 ~
                        c'8 ~
                        c'16
                        c'16 ~
                        c'8
                    }
                }
            }
        >>
        '''
        ), format(score)


def test_quantizationtools_Quantizer___call___05():

    q_schema = quantizationtools.BeatwiseQSchema(
        {'search_tree': quantizationtools.UnweightedSearchTree({2: None})},
        {'search_tree': quantizationtools.UnweightedSearchTree({3: None})},
        {'search_tree': quantizationtools.UnweightedSearchTree({5: None})},
        {'search_tree': quantizationtools.UnweightedSearchTree({7: None})},
        )
    milliseconds = [250, 250, 250, 250] * 4
    q_events = quantizationtools.QEventSequence.from_millisecond_durations(
        milliseconds)
    attack_point_optimizer = quantizationtools.NullAttackPointOptimizer()
    quantizer = quantizationtools.Quantizer()

    result = quantizer(
        q_events,
        q_schema=q_schema,
        attack_point_optimizer=attack_point_optimizer,
        )
    staff = scoretools.Staff([result], context_name='RhythmicStaff')
    score = Score([staff])

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new RhythmicStaff {
                \new Voice {
                    \grace {
                        c'16
                    }
                    \tempo 4=60
                    c'8
                    \grace {
                        c'16
                    }
                    c'8
                    \times 2/3 {
                        c'8
                        \grace {
                            c'16
                        }
                        c'8
                        c'8
                    }
                    \times 4/5 {
                        c'16
                        c'16
                        c'16 ~
                        c'16
                        c'16
                    }
                    \times 4/7 {
                        c'16 ~
                        c'16
                        c'16
                        c'16 ~
                        c'16
                        c'16 ~
                        c'16
                    }
                }
            }
        >>
        '''
        ), format(score)



# -*- coding: utf-8 -*-
from abjad import *
import os


def test_systemtools_IOManager_save_last_ly_as_01():

    abjad_output_directory = abjad_configuration.abjad_output_directory
    lilypond_files = [x for x in os.listdir(abjad_output_directory)
        if x.endswith('.ly')]
    if not lilypond_files:
        note = Note("c'4")
        persist(note).as_ly()

    systemtools.IOManager.save_last_ly_as('tmp_foo.ly')
    assert os.path.exists('tmp_foo.ly')

    os.remove('tmp_foo.ly')
    assert not os.path.exists('tmp_foo.ly')



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmmakertools_TaleaRhythmMaker_tie_split_notes_01():

    talea = rhythmmakertools.Talea(
        counts=[5],
        denominator=16,
        )
    rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        )

    divisions = [(2, 8), (2, 8), (2, 8), (2, 8)]
    selections = rhythm_maker(divisions)

    selections = Sequence(selections).flatten()
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    measures = mutate(staff).replace_measure_contents(selections)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 2/8
                c'4 ~
            }
            {
                c'16 [
                c'8. ~ ]
            }
            {
                c'8 [
                c'8 ~ ]
            }
            {
                c'8. [
                c'16 ]
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_rhythmmakertools_TaleaRhythmMaker_tie_split_notes_02():

    talea = rhythmmakertools.Talea(
        counts=[5],
        denominator=16,
        )
    rhythm_maker = rhythmmakertools.TaleaRhythmMaker(
        talea=talea,
        )

    divisions = [(3, 16), (5, 8), (4, 8), (7, 16)]
    selections = rhythm_maker(divisions)

    selections = Sequence(selections).flatten()
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)
    measures = mutate(staff).replace_measure_contents(selections)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 3/16
                c'8. ~
            }
            {
                \time 5/8
                c'8
                c'4 ~
                c'16 [
                c'8. ~ ]
            }
            {
                \time 4/8
                c'8
                c'4 ~
                c'16 [
                c'16 ~ ]
            }
            {
                \time 7/16
                c'4
                c'8.
            }
        }
        '''
        )

    assert inspect_(staff).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.pitchtools import PitchSegment


def test_pitchtools_PitchSegment___repr___01():
    r'''Named pitch segment repr is evaluable.
    '''

    ncps = ['bf', 'bqf', "fs'", "g'", 'bqf', "g'"]
    named_pitch_segment_1 = pitchtools.PitchSegment(ncps)
    named_pitch_segment_2 = eval(repr(named_pitch_segment_1))

    r'''
    PitchSegment("bf bqf fs' g' bqf g'")
    '''

    assert isinstance(named_pitch_segment_1, pitchtools.PitchSegment)
    assert isinstance(named_pitch_segment_2, pitchtools.PitchSegment)



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TreeNode_depth_01():

    container = datastructuretools.TreeContainer()
    assert container.depth == 0

    leaf = datastructuretools.TreeNode()
    assert leaf.depth == 0

    container.append(leaf)
    assert leaf.depth == 1

    subcontainer = datastructuretools.TreeContainer()
    assert subcontainer.depth == 0

    container.append(subcontainer)
    assert subcontainer.depth == 1

    subcontainer.append(leaf)
    assert leaf.depth == 2

    subsubcontainer = datastructuretools.TreeContainer()
    assert subsubcontainer.depth == 0

    subcontainer.append(subsubcontainer)
    assert subsubcontainer.depth == 2

    subsubcontainer.append(leaf)
    assert leaf.depth == 3



# -*- coding: utf-8 -*-
import platform
import unittest
from abjad.tools import abjadbooktools
from abjad.tools import stringtools


@unittest.skipIf(
    platform.python_implementation() != 'CPython',
    'Only for CPython',
    )
class TestLaTeXDocumentHandler(unittest.TestCase):

    maxDiff = None

    def test_abjadextract_1(self):
        input_file_contents = [
            '',
            '\\begin{comment}',
            '<abjadextract abjad.tools.abjadbooktools:example_function \>',
            '\\end{comment}',
            '',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler()
        input_blocks = document_handler.collect_input_blocks(input_file_contents)
        code_block = tuple(input_blocks.values())[0]
        assert code_block.executed_lines == (
            'from abjad.tools.abjadbooktools import example_function',
            )
        assert code_block.input_file_contents == (
            'def example_function(argument):',
            "    r'''This is a multiline docstring.",
            '',
            '    This is the third line of the docstring.',
            "    '''",
            '    # This is a comment.',
            "    print('Entering example function.')",
            '    try:',
            '        argument = argument + 1',
            '    except TypeError:',
            "        print('Wrong type!')",
            '    print(argument)',
            "    print('Leaving example function.')",
            )

    def test_abjadextract_2(self):
        input_file_contents = [
            '\\begin{comment}',
            '<abjadextract abjad.tools.abjadbooktools:example_function \>',
            '\\end{comment}',
            '',
            '\\begin{comment}',
            '<abjad>[allow_exceptions=true]',
            "example_function('foo')",
            '</abjad>',
            '\\end{comment}',
            '',
            '\\begin{comment}',
            '<abjad>[allow_exceptions=true]',
            "example_function(23)",
            '</abjad>',
            '\\end{comment}',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        rebuilt_source = document_handler(return_source=True)
        assert rebuilt_source == stringtools.normalize(
            """
            \\begin{comment}
            <abjadextract abjad.tools.abjadbooktools:example_function \\>
            \\end{comment}

            %%% ABJADBOOK START %%%
            \\begin{lstlisting}
            def example_function(argument):
                r'''This is a multiline docstring.

                This is the third line of the docstring.
                '''
                # This is a comment.
                print('Entering example function.')
                try:
                    argument = argument + 1
                except TypeError:
                    print('Wrong type!')
                print(argument)
                print('Leaving example function.')
            \\end{lstlisting}
            %%% ABJADBOOK END %%%

            \\begin{comment}
            <abjad>[allow_exceptions=true]
            example_function('foo')
            </abjad>
            \\end{comment}

            %%% ABJADBOOK START %%%
            \\begin{lstlisting}
            >>> example_function('foo')
            Entering example function.
            Wrong type!
            foo
            Leaving example function.
            \\end{lstlisting}
            %%% ABJADBOOK END %%%

            \\begin{comment}
            <abjad>[allow_exceptions=true]
            example_function(23)
            </abjad>
            \\end{comment}

            %%% ABJADBOOK START %%%
            \\begin{lstlisting}
            >>> example_function(23)
            Entering example function.
            24
            Leaving example function.
            \\end{lstlisting}
            %%% ABJADBOOK END %%%
            """,
            )

    def test_abjadextract_3(self):
        input_file_contents = [
            '\\begin{comment}',
            '<abjadextract abjad.tools.abjadbooktools:example_function \>[hide=true]',
            '\\end{comment}',
            '',
            '\\begin{comment}',
            '<abjad>[allow_exceptions=true]',
            "example_function('foo')",
            '</abjad>',
            '\\end{comment}',
            '',
            '\\begin{comment}',
            '<abjad>[allow_exceptions=true]',
            "example_function(23)",
            '</abjad>',
            '\\end{comment}',
            ]
        document_handler = abjadbooktools.LaTeXDocumentHandler(
            input_file_contents=input_file_contents,
            )
        rebuilt_source = document_handler(return_source=True)
        assert rebuilt_source == stringtools.normalize(
            """
            \\begin{comment}
            <abjadextract abjad.tools.abjadbooktools:example_function \\>[hide=true]
            \\end{comment}

            \\begin{comment}
            <abjad>[allow_exceptions=true]
            example_function('foo')
            </abjad>
            \\end{comment}

            %%% ABJADBOOK START %%%
            \\begin{lstlisting}
            >>> example_function('foo')
            Entering example function.
            Wrong type!
            foo
            Leaving example function.
            \\end{lstlisting}
            %%% ABJADBOOK END %%%

            \\begin{comment}
            <abjad>[allow_exceptions=true]
            example_function(23)
            </abjad>
            \\end{comment}

            %%% ABJADBOOK START %%%
            \\begin{lstlisting}
            >>> example_function(23)
            Entering example function.
            24
            Leaving example function.
            \\end{lstlisting}
            %%% ABJADBOOK END %%%
            """,
            )



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_PhrasingSlur_direction_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    slur = spannertools.PhrasingSlur(direction=Up)
    attach(slur, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 ^ \(
            d'8
            e'8
            f'8 \)
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_spannertools_PhrasingSlur_direction_02():

    staff = Staff("c'8 d'8 e'8 f'8")
    slur = spannertools.PhrasingSlur(direction=Down)
    attach(slur, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 _ \(
            d'8
            e'8
            f'8 \)
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchRange___contains___01():
    r'''Closed / closed range.
    '''

    range_ = pitchtools.PitchRange('[A0, C8]')
    assert -99 not in range_
    assert -39 in range_
    assert 0 in range_
    assert 48 in range_
    assert 99 not in range_


def test_pitchtools_PitchRange___contains___02():
    r'''Closed / open range.
    '''

    range_ = pitchtools.PitchRange('[A0, C8)')
    assert -99 not in range_
    assert -39 in range_
    assert 0 in range_
    assert 48 not in range_
    assert 99 not in range_


def test_pitchtools_PitchRange___contains___03():
    r'''Closed / infinite range.
    '''

    #range_ = pitchtools.PitchRange((-39, 'inclusive'), None)
    range_ = pitchtools.PitchRange('[-39, +inf]')
    assert -99 not in range_
    assert -39 in range_
    assert 0 in range_
    assert 48 in range_
    assert 99 in range_


def test_pitchtools_PitchRange___contains___04():
    r'''Open / closed range.
    '''

    range_ = pitchtools.PitchRange('(A0, C8]')
    assert -99 not in range_
    assert -39 not in range_
    assert 0 in range_
    assert 48 in range_
    assert 99 not in range_


def test_pitchtools_PitchRange___contains___05():
    r'''Open / open range.
    '''

    range_ = pitchtools.PitchRange('(-39, 48)')
    assert -99 not in range_
    assert -39 not in range_
    assert 0 in range_
    assert 48 not in range_
    assert 99 not in range_


def test_pitchtools_PitchRange___contains___06():
    r'''Open / infinite range.
    '''

    #range_ = pitchtools.PitchRange((-39, 'exclusive'), None)
    range_ = pitchtools.PitchRange('(-39, +inf]')
    assert -99 not in range_
    assert -39 not in range_
    assert 0 in range_
    assert 48 in range_
    assert 99 in range_


def test_pitchtools_PitchRange___contains___07():
    r'''Infinite / closed range.
    '''

    #range_ = pitchtools.PitchRange(None, (48, 'inclusive'))
    range_ = pitchtools.PitchRange('[-inf, C8]')
    assert -99 in range_
    assert -39 in range_
    assert 0 in range_
    assert 48 in range_
    assert 99 not in range_


def test_pitchtools_PitchRange___contains___08():
    r'''Infinite / open range.
    '''

    #range_ = pitchtools.PitchRange(None, (48, 'exclusive'))
    range_ = pitchtools.PitchRange('[-inf, C8)')
    assert -99 in range_
    assert -39 in range_
    assert 0 in range_
    assert 48 not in range_
    assert 99 not in range_


def test_pitchtools_PitchRange___contains___09():
    r'''Infinite / infinite range.
    '''

    #range_ = pitchtools.PitchRange(None, None)
    range_ = pitchtools.PitchRange('[-inf, +inf]')
    assert -99 in range_
    assert -39 in range_
    assert 0 in range_
    assert 48 in range_
    assert 99 in range_


def test_pitchtools_PitchRange___contains___10():
    r'''Chord containement.
    '''

    #range_ = pitchtools.PitchRange((-39, 'inclusive'), (48, 'inclusive'))
    range_ = pitchtools.PitchRange('[-39, 48]')
    assert Chord([-99, -98, -97], (1, 4)) not in range_
    assert Chord([-39, -38, -37], (1, 4)) in range_
    assert Chord([0, 2, 3], (1, 4)) in range_
    assert Chord([46, 47, 48], (1, 4)) in range_
    assert Chord([48, 49, 50], (1, 4)) not in range_


def test_pitchtools_PitchRange___contains___11():
    r'''Note containement.
    '''

    #range_ = pitchtools.PitchRange((-39, 'inclusive'), (48, 'inclusive'))
    range_ = pitchtools.PitchRange('[-39, 48]')
    assert Note(-99, (1, 4)) not in range_
    assert Note(-39, (1, 4)) in range_
    assert Note(  0, (1, 4)) in range_
    assert Note( 48, (1, 4)) in range_
    assert Note( 99, (1, 4)) not in range_


def test_pitchtools_PitchRange___contains___12():
    r'''Rest and skip containement.
    '''

    #range_ = pitchtools.PitchRange((-39, 'inclusive'), (48, 'inclusive'))
    range_ = pitchtools.PitchRange('[-39, 48]')
    assert Rest((1, 4)) in range_
    assert scoretools.Skip((1, 4)) in range_


def test_pitchtools_PitchRange___contains___13():
    r'''Iterable containment.
    '''

    pitch_numbers = range(10)

    assert pitch_numbers in pitchtools.PitchRange.from_pitches(-39, 48)
    assert not pitch_numbers in pitchtools.PitchRange.from_pitches(36, 48)


def test_pitchtools_PitchRange___contains___14():
    r'''Works with transposed pitches.
    '''

    staff = Staff("<c''' e'''>4 <d''' fs'''>4")
    glockenspiel = instrumenttools.Glockenspiel()
    attach(glockenspiel, staff)
    instrumenttools.transpose_from_sounding_pitch_to_written_pitch(staff)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.instrumentName = \markup { Glockenspiel }
            \set Staff.shortInstrumentName = \markup { Gkspl. }
            <c' e'>4
            <d' fs'>4
        }
        '''
        )

    assert staff[0] in glockenspiel.pitch_range
    assert staff[1] in glockenspiel.pitch_range
    assert staff in glockenspiel.pitch_range

    assert not Note("c'4") in glockenspiel.pitch_range


def test_pitchtools_PitchRange___contains___15():
    r'''Unpitched notes and chords are evaluated as in-range by definition.
    '''

    staff = Staff("c'4 d'4 c4 d4")
    flute = instrumenttools.Flute()
    attach(flute, staff)
    attach('unpitched', staff[2])
    attach('unpitched', staff[3])
    override(staff[2]).note_head.style = 'cross'
    override(staff[3]).note_head.style = 'cross'

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \set Staff.instrumentName = \markup { Flute }
            \set Staff.shortInstrumentName = \markup { Fl. }
            c'4
            d'4
            \once \override NoteHead.style = #'cross
            c4
            \once \override NoteHead.style = #'cross
            d4
        }
        '''
        )

    assert staff in flute.pitch_range



# -*- coding: utf-8 -*-
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools.topleveltools import iterate


def notes_and_chords_are_on_expected_clefs(
    argument,
    percussion_clef_is_allowed=True,
    ):
    r'''Is true when notes and chords in `argument` are on expected clefs.

    ..  todo:: Move to WellformednessManager.

    ..  container:: example

        **Example 1.** Expected clef:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> clef = Clef(name='treble')
            >>> attach(clef, staff)
            >>> violin = instrumenttools.Violin()
            >>> attach(violin, staff)
            >>> show(staff) # doctest: +SKIP

        ::

            >>> instrumenttools.notes_and_chords_are_on_expected_clefs(staff)
            True

    ..  container:: example

        **Example 2.** Unexpected clef:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> clef = Clef(name='alto')
            >>> attach(clef, staff)
            >>> violin = instrumenttools.Violin()
            >>> attach(violin, staff)
            >>> show(staff) # doctest: +SKIP

        ::

            >>> instrumenttools.notes_and_chords_are_on_expected_clefs(staff)
            False

    ..  container:: example

        **Example 3.** Allows percussion clef:

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> clef = Clef(name='percussion')
            >>> attach(clef, staff)
            >>> violin = instrumenttools.Violin()
            >>> attach(violin, staff)
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \clef "percussion"
                \set Staff.instrumentName = \markup { Violin }
                \set Staff.shortInstrumentName = \markup { Vn. }
                c'8
                d'8
                e'8
                f'8
            }

        ::

            >>> instrumenttools.notes_and_chords_are_on_expected_clefs(
            ...     staff, percussion_clef_is_allowed=True)
            True

    ..  container:: example

        **Example 4.** Forbids percussion clef:

        ::

            >>> instrumenttools.notes_and_chords_are_on_expected_clefs(
            ...     staff, percussion_clef_is_allowed=False)
            False

    Returns true or false.
    '''
    from abjad.tools import instrumenttools
    prototype = (scoretools.Note, scoretools.Chord)
    for note_or_chord in iterate(argument).by_class(prototype):
        instrument = note_or_chord._get_effective(instrumenttools.Instrument)
        if not instrument:
            return False
        clef = note_or_chord._get_effective(indicatortools.Clef)
        if not clef:
            return False
        if clef == indicatortools.Clef(name='percussion'):
            if percussion_clef_is_allowed:
                return True
            else:
                return False
        if clef not in instrument.allowable_clefs:
            return False
    else:
        return True


# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Container___getitem___01():
    r'''Get one container component with positive index.
    '''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    voice = Voice(notes)

    assert voice[0] is notes[0]
    assert voice[1] is notes[1]
    assert voice[2] is notes[2]
    assert voice[3] is notes[3]


def test_scoretools_Container___getitem___02():
    r'''Get one container component with negative index.
    '''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    voice = Voice(notes)

    assert voice[-1] is notes[3]
    assert voice[-2] is notes[2]
    assert voice[-3] is notes[1]
    assert voice[-4] is notes[0]


def test_scoretools_Container___getitem___03():
    r'''Get slice from container.
    '''

    notes = [Note("c'8"), Note("d'8"), Note("e'8"), Note("f'8")]
    voice = Voice(notes)

    assert voice[:1] == notes[:1]
    assert voice[:2] == notes[:2]
    assert voice[:3] == notes[:3]
    assert voice[:4] == notes[:4]


def test_scoretools_Container___getitem___04():
    r'''Bad index raises IndexError.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")

    assert pytest.raises(IndexError, 'voice[99]')


def test_scoretools_Container___getitem___05():
    r'''Get one named component in subtree rooted at container.
    '''

    template = templatetools.StringQuartetScoreTemplate()
    score = template()

    assert score['First Violin Staff'].name == 'First Violin Staff'
    assert score['First Violin Voice'].name == 'First Violin Voice'


def test_scoretools_Container___getitem___06():
    r'''Bad name raises exception.
    '''

    template = templatetools.StringQuartetScoreTemplate()
    score = template()

    assert pytest.raises(Exception, "score['Foo']")


def test_scoretools_Container___getitem___07():
    r'''Duplicate named contexts raise exception.
    '''

    template = templatetools.StringQuartetScoreTemplate()
    score = template()

    assert score['First Violin Voice'].name == 'First Violin Voice'

    score['Cello Staff'].append(Voice(name='First Violin Voice'))

    assert pytest.raises(Exception, "score['First Violin Voice']")

    extra_first_violin_voice = score['Cello Staff'].pop()

    assert score['First Violin Voice'].name == 'First Violin Voice'
    assert score['First Violin Voice'] is not extra_first_violin_voice



# -*- coding: utf-8 -*-
import abc
from abjad.tools.abctools import AbjadObject


class JobHandler(AbjadObject):
    r'''Abstact job handler class from which concrete job handlers inherit.

    ``JobHandlers`` control how ``QuantizationJob`` instances are
    processed by the ``Quantizer``, either serially or in parallel.
    '''

    ### INITIALIZER ###

    def __init__(self):
        pass

    ### SPECIAL METHODS ###

    @abc.abstractmethod
    def __call__(self, jobs):
        r'''Calls job handler.
        '''
        raise NotImplementedError



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Accidental___cmp___01():
    r'''Accidentals equal by string compare equally.
    '''
    k1, k2 = pitchtools.Accidental('s'), pitchtools.Accidental('s')
    assert k1 == k2
    assert not k1 != k2
    assert not k1 > k2
    assert k1 >= k2
    assert not k1 < k2
    assert k1 <= k2


def test_pitchtools_Accidental___cmp___02():
    r'''Accidentals unequal by string compare unequally.
    '''
    k1, k2 = pitchtools.Accidental('s'), pitchtools.Accidental('f')
    assert not k1 == k2
    assert k1 != k2
    assert k1 > k2
    assert k1 >= k2
    assert not k1 < k2
    assert not k1 <= k2



# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_MeasurewiseQSchema___init___01():

    item_a = quantizationtools.MeasurewiseQSchemaItem(search_tree=quantizationtools.UnweightedSearchTree({2: None}))
    item_b = quantizationtools.MeasurewiseQSchemaItem(tempo=((1, 4), 76))
    item_c = quantizationtools.MeasurewiseQSchemaItem(time_signature=(3, 4))
    item_d = quantizationtools.MeasurewiseQSchemaItem(use_full_measure=True)

    schema = quantizationtools.MeasurewiseQSchema(
        {2: item_a, 4: item_b, 7: item_c, 8: item_d},
        search_tree=quantizationtools.UnweightedSearchTree({3: None}),
        tempo=((1, 8), 58),
        time_signature=(5, 8),
        use_full_measure=False,
        )

    assert len(schema.items) == 4
    assert schema.search_tree == quantizationtools.UnweightedSearchTree({3: None})
    assert schema.tempo == Tempo((1, 8), 58)
    assert schema.time_signature == TimeSignature((5, 8))
    assert schema.use_full_measure == False


def test_quantizationtools_MeasurewiseQSchema___init___02():

    schema = quantizationtools.MeasurewiseQSchema()

    assert len(schema.items) == 0
    assert schema.search_tree == quantizationtools.UnweightedSearchTree()
    assert schema.tempo == Tempo((1, 4), 60)
    assert schema.time_signature == TimeSignature((4, 4))
    assert schema.use_full_measure == False



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_VerticalMoment___len___01():

    score = Score(
        r'''
        \new Staff {
            \times 4/3 {
                d''8
                c''8
                b'8
            }
        }
        \new PianoStaff <<
            \new Staff {
                a'4
                g'4
            }
            \new Staff {
                \clef "bass"
                f'8
                e'8
                d'8
                c'8
            }
        >>
        '''
        )

    staff_group = score[1]

    vertical_moment = inspect_(score).get_vertical_moment_at(Offset(1, 8))
    "VerticalMoment(Score<<2>>, Staff{1}, {@ 3:4 d''8, c''8, b'8 @}, d''8, PianoStaff<<2>>, Staff{2}, a'4, Staff{4}, e'8)"
    assert len(vertical_moment) == 9

    vertical_moment = inspect_(score[0]).get_vertical_moment_at(Offset(1, 8))
    "VerticalMoment(Staff{1}, {@ 3:4 d''8, c''8, b'8 @}, d''8)"
    assert len(vertical_moment) == 3

    vertical_moment = inspect_(staff_group).get_vertical_moment_at(Offset(1, 8))
    "VerticalMoment(PianoStaff<<2>>, Staff{2}, a'4, Staff{4}, e'8)"
    assert len(vertical_moment) == 5

    vertical_moment = inspect_(staff_group[0]).get_vertical_moment_at(Offset(1, 8))
    "VerticalMoment(Staff{2}, a'4)"
    assert len(vertical_moment) == 2

    vertical_moment = inspect_(staff_group[1]).get_vertical_moment_at(Offset(1, 8))
    "VerticalMoment(Staff{2}, e'8)"
    assert len(vertical_moment) == 2



# -*- coding: utf-8 -*-
from abjad.tools.schemetools.Scheme import Scheme


class SchemeAssociativeList(Scheme):
    '''Abjad model of Scheme associative list.

    ..  container:: example

        ::

            >>> scheme_alist = schemetools.SchemeAssociativeList(
            ...     ('space', 2),
            ...     ('padding', 0.5),
            ...     )
            >>> scheme_alist
            SchemeAssociativeList(SchemePair('space', 2), SchemePair('padding', 0.5), quoting="'")

        ::

            >>> print(format(scheme_alist))
            #'((space . 2) (padding . 0.5))

    Scheme associative lists are immutable.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### INITIALIZER ###

    def __init__(self, *arguments, **keywords):
        from abjad.tools import schemetools
        args_as_pairs = []
        for arg in arguments:
            if not isinstance(arg, (tuple, schemetools.SchemePair)):
                message = 'must be Python pair or Scheme pair: "%s".'
                raise TypeError(message % arg)
            arg_as_pair = schemetools.SchemePair(*arg)
            args_as_pairs.append(arg_as_pair)
        Scheme.__init__(self, *args_as_pairs, **{'quoting': "'"})



# -*- coding: utf-8 -*-
import abc
import bisect
import copy
from abjad.tools import mathtools
from abjad.tools.abctools import AbjadObject


class QSchema(AbjadObject):
    r'''The *schema* for a quantization run.

    ``QSchema`` allows for the specification of quantization settings
    diachronically, at any time-step of the quantization process.

    In practice, this provides a means for the composer to change the tempo,
    search-tree, time-signature etc., effectively creating a template into
    which quantized rhythms can be "poured", without yet knowing what those
    rhythms might be, or even how much time the ultimate result will take.
    Like Abjad indicators the settings made at any given time-step via
    a ``QSchema`` instance are understood to persist until changed.

    All concrete ``QSchema`` subclasses strongly implement default values for
    all of their parameters.

    `QSchema` is abstract.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_items',
        '_lookups',
        )

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, *arguments, **keywords):
        if 1 == len(arguments) and isinstance(arguments[0], type(self)):
            items = copy.copy(arguments[0].items)
        elif 1 == len(arguments) and isinstance(arguments[0], dict):
            items = list(arguments[0].items())
            if mathtools.all_are_pairs_of_types(items, int, dict):
                items = [(x, self.item_class(**y)) for x, y in items]
            assert mathtools.all_are_pairs_of_types(
                items, int, self.item_class)
            items = dict(items)
        elif mathtools.all_are_pairs_of_types(arguments, int, self.item_class):
            items = dict(arguments)
        elif mathtools.all_are_pairs_of_types(arguments, int, dict):
            items = [(x, self.item_class(**y)) for x, y in arguments]
            items = dict(items)
        elif all(isinstance(x, self.item_class) for x in arguments):
            items = [(i, x) for i, x in enumerate(arguments)]
            items = dict(items)
        elif all(isinstance(x, dict) for x in arguments):
            items = [(i, self.item_class(**x)) for i, x in enumerate(arguments)]
            items = dict(items)
        else:
            raise ValueError
        if items:
            assert 0 <= min(items)
        self._items = dict(items)
        self._lookups = self._create_lookups()

    ### SPECIAL METHODS ###

    def __call__(self, duration):
        r'''Calls QSchema on `duration`.
        '''
        target_items = []
        idx, current_offset = 0, 0
        while current_offset < duration:
            lookup = self[idx]
            lookup['offset_in_ms'] = current_offset
            target_item = self.target_item_class(**lookup)
            target_items.append(target_item)
            current_offset += target_item.duration_in_ms
            idx += 1
        return self.target_class(target_items)

    def __format__(self, format_specification=''):
        r'''Formats q-event.

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __getitem__(self, argument):
        r'''Gets item or slice identified by `argument`.
        '''
        assert isinstance(argument, int) and 0 <= argument
        result = {}
        for field in self._lookups:
            lookup = self._lookups[field].get(argument)
            if lookup is not None:
                result[field] = lookup
            else:
                keys = sorted(self._lookups[field].keys())
                idx = bisect.bisect(keys, argument)
                if len(keys) == idx:
                    key = keys[-1]
                elif argument < keys[idx]:
                    key = keys[idx - 1]
                result[field] = self._lookups[field][key]
        return result

    ### PRIVATE METHODS ###

    def _create_lookups(self):
        names = self._keyword_argument_names
        lookups = {}
        for name in names:
            lookups[name] = {0: getattr(self, name)}
            for position, item in self.items.items():
                value = getattr(item, name)
                if value is not None:
                    lookups[name][position] = value
            lookups[name] = dict(lookups[name])
        return dict(lookups)

    ### PUBLIC PROPERTIES ###

    @abc.abstractproperty
    def item_class(self):
        r'''The schema's item class.
        '''
        raise NotImplementedError

    @property
    def items(self):
        r'''The item dictionary.
        '''
        return self._items

    @property
    def search_tree(self):
        r'''The default search tree.
        '''
        return self._search_tree

    @abc.abstractproperty
    def target_class(self):
        r'''The schema's target class.
        '''
        raise NotImplementedError

    @abc.abstractproperty
    def target_item_class(self):
        r'''The schema's target class' item class.
        '''
        raise NotImplementedError

    @property
    def tempo(self):
        r'''The default tempo.
        '''
        return self._tempo



# -*- coding: utf-8 -*-
from abjad import *


def test_timespantools_Timespan_stops_before_timespan_starts_01():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, -5)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_02():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 0)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_03():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 5)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_04():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 15)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_05():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 25)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_06():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 10)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_07():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 15)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_08():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 10)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_09():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 15)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_10():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 25)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_11():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 25)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_12():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(15, 25)
    assert not timespan_1.stops_before_timespan_starts(timespan_2)

def test_timespantools_Timespan_stops_before_timespan_starts_13():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(20, 25)
    assert timespan_1.stops_before_timespan_starts(timespan_2)



# -*- coding: utf-8 -*-
from abjad import *


def test_schemetools_Scheme_format_scheme_value_01():

    assert schemetools.Scheme.format_scheme_value(1) == '1'
    assert schemetools.Scheme.format_scheme_value(True) == '#t'
    assert schemetools.Scheme.format_scheme_value(False) == '#f'
    assert schemetools.Scheme.format_scheme_value('foo bar') == '"foo bar"'
    assert schemetools.Scheme.format_scheme_value('baz') == 'baz'
    assert schemetools.Scheme.format_scheme_value([1, 2, 3]) == '(1 2 3)'



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_agenttools_InspectionAgent_get_indicator_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    attach('color', staff)

    assert inspect_(staff).get_indicator('color') == 'color'
    assert inspect_(staff[0]).get_indicator('color') is None
    assert inspect_(staff[1]).get_indicator('color') is None
    assert inspect_(staff[2]).get_indicator('color') is None
    assert inspect_(staff[3]).get_indicator('color') is None


def test_agenttools_InspectionAgent_get_indicator_02():

    staff = Staff("c'8 d'8 e'8 f'8")
    attach('color', staff)

    assert inspect_(staff).get_indicator(str) == 'color'
    assert inspect_(staff[0]).get_indicator(str) is None
    assert inspect_(staff[1]).get_indicator(str) is None
    assert inspect_(staff[2]).get_indicator(str) is None
    assert inspect_(staff[3]).get_indicator(str) is None


def test_agenttools_InspectionAgent_get_indicator_03():

    note = Note("c'8")
    annotation = indicatortools.Annotation('special information')
    attach(annotation, note)

    assert inspect_(note).get_indicator(indicatortools.Annotation) is annotation


def test_agenttools_InspectionAgent_get_indicator_04():

    note = Note("c'8")

    assert inspect_(note).get_indicator(indicatortools.Annotation) is None


def test_agenttools_InspectionAgent_get_indicator_05():

    note = Note("c'8")
    annotation = indicatortools.Annotation('special information')
    attach(annotation, note)
    annotation = indicatortools.Annotation('more special information')
    attach(annotation, note)

    statement = 'inspect_(note).get_indicator(indicatortools.Annotation)'
    assert pytest.raises(Exception, statement)


def test_agenttools_InspectionAgent_get_indicator_06():

    note = Note("c'8")
    articulation = Articulation('staccato')
    attach(articulation, note)

    assert inspect_(note).get_indicator(Articulation) is articulation


def test_agenttools_InspectionAgent_get_indicator_07():

    note = Note("c'8")

    assert inspect_(note).get_indicator(Articulation) is None


def test_agenttools_InspectionAgent_get_indicator_08():

    note = Note("c'8")
    articulation = Articulation('staccato')
    attach(articulation, note)
    articulation = Articulation('marcato')
    attach(articulation, note)

    statement = 'inspect_(note).get_indicator(Articulation)'
    assert pytest.raises(Exception, statement)


def test_agenttools_InspectionAgent_get_indicator_09():

    note = Note("c'8")
    command = indicatortools.LilyPondCommand('stemUp')
    attach(command, note)

    result = inspect_(note).get_indicator(indicatortools.LilyPondCommand)
    assert result is command


def test_agenttools_InspectionAgent_get_indicator_10():

    note = Note("c'8")

    assert inspect_(note).get_indicator(indicatortools.LilyPondCommand) is None


def test_agenttools_InspectionAgent_get_indicator_11():

    note = Note("c'8")
    command = indicatortools.LilyPondCommand('stemUp')
    attach(command, note)
    command = indicatortools.LilyPondCommand('slurUp')
    attach(command, note)

    statement = 'inspect_(note).get_indicator(indicatortools.LilyPondCommand)'
    assert pytest.raises(Exception, statement)


def test_agenttools_InspectionAgent_get_indicator_12():

    staff = Staff("c'8 d'8 e'8 f'8")
    slur = Slur()
    attach(slur, staff[:])
    command_1 = indicatortools.LilyPondCommand('slurDotted')
    attach(command_1, staff[0])
    command_2 = indicatortools.LilyPondCommand('slurUp')
    attach(command_2, staff[0])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \slurDotted
            \slurUp
            c'8 (
            d'8
            e'8
            f'8 )
        }
        '''
        )

    indicators = inspect_(staff[0]).get_indicators(
        indicatortools.LilyPondCommand)

    assert command_1 in indicators
    assert command_2 in indicators
    assert len(indicators) == 2


def test_agenttools_InspectionAgent_get_indicator_13():

    note = Note("c'8")
    comment = indicatortools.LilyPondComment('comment')
    attach(comment, note)

    indicator = inspect_(note).get_indicator(indicatortools.LilyPondComment)
    assert indicator is comment


def test_agenttools_InspectionAgent_get_indicator_14():

    note = Note("c'8")

    assert inspect_(note).get_indicator(indicatortools.LilyPondComment) is None


def test_agenttools_InspectionAgent_get_indicator_15():

    note = Note("c'8")
    comment = indicatortools.LilyPondComment('comment')
    attach(comment, note)
    comment = indicatortools.LilyPondComment('another comment')
    attach(comment, note)

    statement = 'inspect_(note).get_indicator(indicatortools.LilyPondComment)'
    assert pytest.raises(Exception, statement)


def test_agenttools_InspectionAgent_get_indicator_16():

    note = Note("c'8")

    assert inspect_(note).get_indicator() is None


def test_agenttools_InspectionAgent_get_indicator_17():

    note = Note("c'4")
    stem_tremolo = indicatortools.StemTremolo(16)
    attach(stem_tremolo, note)
    stem_tremolo = inspect_(note).get_indicator(indicatortools.StemTremolo)

    assert stem_tremolo is stem_tremolo


def test_agenttools_InspectionAgent_get_indicator_18():

    staff = Staff("c'8 d'8 e'8 f'8")
    violin = instrumenttools.Violin()
    attach(violin, staff)

    indicator = inspect_(staff).get_indicator(instrumenttools.Instrument)

    assert indicator is violin


def test_agenttools_InspectionAgent_get_indicator_19():

    measure = Measure((4, 8), "c'8 d'8 e'8 f'8")
    indicator = inspect_(measure).get_indicator(TimeSignature)

    assert indicator == TimeSignature((4, 8))


# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_UnweightedSearchTree__find_leaf_subdivisions_01():

    definition = {
        2: {
            2: {
                2: None
            },
            3: None
        },
        5: None
    }
    search_tree = quantizationtools.UnweightedSearchTree(definition)

    assert search_tree._find_leaf_subdivisions((1, (1, 2))) == ((1, 1), (1, 1, 1))
    assert search_tree._find_leaf_subdivisions((1, (1, 2), (1, 2))) == ((1, 1),)
    assert search_tree._find_leaf_subdivisions((1, (1, 2), (1, 2), (1, 2))) == ()
    assert search_tree._find_leaf_subdivisions((1, (1, 2), (1, 3))) == ()
    assert search_tree._find_leaf_subdivisions((1, (1, 5))) == ()



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Measure_should_scale_contents_01():

    tuplet = Tuplet((2, 3), "c'8 d'8 e'8 f'8 g'8")
    measure = Measure((5, 12), [tuplet], implicit_scaling=False)

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 5/12
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                c'8
                d'8
                e'8
                f'8
                g'8
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_IterationAgent_by_semantic_voice_01():

    durations = [(3, 8), (5, 16), (5, 16)]
    measures = scoretools.make_spacer_skip_measures(durations)
    time_signature_voice = Voice(measures)
    time_signature_voice.name = 'TimeSignatureVoice'
    time_signature_voice.is_nonsemantic = True
    music_voice = Voice("c'4. d'4 e'16 f'4 g'16")
    music_voice.name = 'MusicVoice'
    staff = Staff([time_signature_voice, music_voice])
    staff.is_simultaneous = True

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff <<
            \context Voice = "TimeSignatureVoice" {
                {
                    \time 3/8
                    s1 * 3/8
                }
                {
                    \time 5/16
                    s1 * 5/16
                }
                {
                    s1 * 5/16
                }
            }
            \context Voice = "MusicVoice" {
                c'4.
                d'4
                e'16
                f'4
                g'16
            }
        >>
        '''
        )

    voices = iterate(staff).by_semantic_voice(reverse=True)
    voices = list(voices)

    assert len(voices) == 1
    assert voices[0] is staff[1]


def test_agenttools_IterationAgent_by_semantic_voice_02():

    durations = [(3, 8), (5, 16), (5, 16)]
    measures = scoretools.make_spacer_skip_measures(durations)
    time_signature_voice = Voice(measures)
    time_signature_voice.name = 'TimeSignatureVoice'
    time_signature_voice.is_nonsemantic = True
    music_voice = Voice("c'4. d'4 e'16 f'4 g'16")
    music_voice.name = 'MusicVoice'
    staff = Staff([time_signature_voice, music_voice])
    staff.is_simultaneous = True

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff <<
            \context Voice = "TimeSignatureVoice" {
                {
                    \time 3/8
                    s1 * 3/8
                }
                {
                    \time 5/16
                    s1 * 5/16
                }
                {
                    s1 * 5/16
                }
            }
            \context Voice = "MusicVoice" {
                c'4.
                d'4
                e'16
                f'4
                g'16
            }
        >>
        '''
        )

    voices = iterate(staff).by_semantic_voice()
    voices = list(voices)

    assert len(voices) == 1
    assert voices[0] is staff[1]



# -*- coding: utf-8 -*-
from abjad.tools.documentationtools.ReSTDirective import ReSTDirective


class ReSTOnlyDirective(ReSTDirective):
    r'''A ReST `only` directive.

    ::

        >>> only = documentationtools.ReSTOnlyDirective(argument='latex')

    ::

        >>> heading = documentationtools.ReSTHeading(
        ...     level=3, text='A LaTeX-Only Heading')
        >>> only.append(heading)
        >>> only
        ReSTOnlyDirective(
            argument='latex',
            children=(
                ReSTHeading(
                    level=3,
                    text='A LaTeX-Only Heading'
                    ),
                )
            )

    ::

        >>> print(only.rest_format)
        .. only:: latex
        <BLANKLINE>
           A LaTeX-Only Heading
           --------------------

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### INITIALIZER ###

    def __init__(self, argument=None, children=None, name=None):
        ReSTDirective.__init__(
            self,
            argument=argument,
            children=children,
            name=name,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def directive(self):
        r'''Returns ``'only'``.
        '''
        return 'only'



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_Selection___illustrate___01():

    staff = Staff("c'4 d'4 e'4 f'4 g'4 a'4 b'4 c''4")
    selection = staff[2:6]
    lilypond_file = selection.__illustrate__()
    score = lilypond_file[Score]

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                e'4
                f'4
                g'4
                a'4
            }
        >>
        '''
        )


def test_selectiontools_Selection___illustrate___02():

    staff = Staff("c'4 d'4 e'4 f'4 g'4 a'4 b'4 c''4")
    attach(Slur(), staff[:])
    selection = staff[2:6]
    lilypond_file = selection.__illustrate__()
    score = lilypond_file[Score]

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                e'4 (
                f'4
                g'4
                a'4 )
            }
        >>
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools import mathtools
from abjad.tools import pitchtools
from abjad.tools import rhythmmakertools
from abjad.tools import selectortools
from abjad.tools import systemtools


def test_systemtools_StorageFormatAgent_get_import_statements_01():
    subject = pitchtools.NamedPitch()
    agent = systemtools.StorageFormatAgent(subject)
    assert agent.get_import_statements() == (
        'from abjad.tools import pitchtools',
        )


def test_systemtools_StorageFormatAgent_get_import_statements_02():
    subject = selectortools.Selector().by_leaf()
    agent = systemtools.StorageFormatAgent(subject)
    assert agent.get_import_statements() == (
        'from abjad.tools import scoretools',
        'from abjad.tools import selectortools',
        )


def test_systemtools_StorageFormatAgent_get_import_statements_03():
    subject = [
        indicatortools.TimeSignature((3, 4)),
        indicatortools.TimeSignature((4, 4)),
        ]
    agent = systemtools.StorageFormatAgent(subject)
    assert agent.get_import_statements() == (
        'from abjad.tools import indicatortools',
        )


def test_systemtools_StorageFormatAgent_get_import_statements_04():
    subject = rhythmmakertools.IncisedRhythmMaker(
        incise_specifier=rhythmmakertools.InciseSpecifier(
            prefix_talea=(1,),
            prefix_counts=(0,),
            suffix_talea=(1,),
            suffix_counts=(1,),
            talea_denominator=16,
            body_ratio=mathtools.Ratio((1,)),
            outer_divisions_only=True,
            ),
        beam_specifier=rhythmmakertools.BeamSpecifier(
            beam_each_division=False,
            beam_divisions_together=False,
            ),
        duration_spelling_specifier=rhythmmakertools.DurationSpellingSpecifier(
            decrease_durations_monotonically=True,
            forbidden_written_duration=durationtools.Duration(1, 2),
            ),
        tuplet_spelling_specifier=rhythmmakertools.TupletSpellingSpecifier(
            avoid_dots=True,
            is_diminution=True,
            simplify_redundant_tuplets=True,
            ),
        )
    agent = systemtools.StorageFormatAgent(subject)
    assert agent.get_import_statements() == (
        'from abjad.tools import durationtools',
        'from abjad.tools import mathtools',
        'from abjad.tools import rhythmmakertools',
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Staff_append_01():
    r'''Append one note.
    '''

    staff = Staff(Note("c'4") * 4)
    staff.append(Note("c'4"))
    assert inspect_(staff).is_well_formed()
    assert len(staff) == 5
    assert staff._contents_duration == Duration(5, 4)


def test_scoretools_Staff_append_02():
    r'''Append one chord.
    '''

    staff = Staff(Note("c'4") * 4)
    staff.append(Chord([2, 3, 4], (1, 4)))
    assert inspect_(staff).is_well_formed()
    assert len(staff) == 5
    assert staff._contents_duration == Duration(5, 4)


def test_scoretools_Staff_append_03():
    r'''Append one tuplet.
    '''

    staff = Staff(Note("c'4") * 4)
    staff.append(scoretools.FixedDurationTuplet(Duration(2, 8), Note(0, (1, 8)) * 3))
    assert inspect_(staff).is_well_formed()
    assert len(staff) == 5
    assert staff._contents_duration == Duration(5, 4)


def test_scoretools_Staff_append_04():
    r'''Empty containers are allowed but not well-formed.
    '''

    staff = Staff(Note("c'4") * 4)
    staff.append(scoretools.FixedDurationTuplet(Duration(2, 8), []))
    assert len(staff) == 5
    assert staff._contents_duration == Duration(5, 4)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_notes_with_multiplied_durations_01():

    notes = scoretools.make_notes_with_multiplied_durations(
        0, Duration(1, 4), [(1, 2), (1, 3), (1, 4), (1, 5)])
    staff = Staff(notes)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'4 * 2
            c'4 * 4/3
            c'4 * 1
            c'4 * 4/5
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedIntervalClass_from_pitch_carriers_01():
    r'''Ascending intervals greater than an octave.
    '''

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(-3), NamedPitch(12))
    assert mcic == pitchtools.NumberedIntervalClass(3)

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(-2), NamedPitch(12))
    assert mcic == pitchtools.NumberedIntervalClass(2)

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(-1), NamedPitch(12))
    assert mcic == pitchtools.NumberedIntervalClass(1)


def test_pitchtools_NumberedIntervalClass_from_pitch_carriers_02():
    r'''Ascending octave.
    '''

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(0), NamedPitch(12))
    assert mcic == pitchtools.NumberedIntervalClass(12)


def test_pitchtools_NumberedIntervalClass_from_pitch_carriers_03():
    r'''Ascending intervals less than an octave.
    '''

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(9), NamedPitch(12))
    assert mcic == pitchtools.NumberedIntervalClass(3)

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(10), NamedPitch(12))
    assert mcic == pitchtools.NumberedIntervalClass(2)

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(11), NamedPitch(12))
    assert mcic == pitchtools.NumberedIntervalClass(1)


def test_pitchtools_NumberedIntervalClass_from_pitch_carriers_04():
    r'''Unison.
    '''

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(12))
    assert mcic == pitchtools.NumberedIntervalClass(0)


def test_pitchtools_NumberedIntervalClass_from_pitch_carriers_05():
    r'''Descending intervals greater than an octave.
    '''

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(-3))
    assert mcic == pitchtools.NumberedIntervalClass(-3)

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(-2))
    assert mcic == pitchtools.NumberedIntervalClass(-2)

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(-1))
    assert mcic == pitchtools.NumberedIntervalClass(-1)


def test_pitchtools_NumberedIntervalClass_from_pitch_carriers_06():
    r'''Descending octave.
    '''

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(0))
    assert mcic == pitchtools.NumberedIntervalClass(-12)


def test_pitchtools_NumberedIntervalClass_from_pitch_carriers_07():
    r'''Descending intervals less than an octave.
    '''

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(9))
    assert mcic == pitchtools.NumberedIntervalClass(-3)

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(10))
    assert mcic == pitchtools.NumberedIntervalClass(-2)

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(11))
    assert mcic == pitchtools.NumberedIntervalClass(-1)


def test_pitchtools_NumberedIntervalClass_from_pitch_carriers_08():
    r'''Quartertones.
    '''

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(-2.5))
    assert mcic == pitchtools.NumberedIntervalClass(-2.5)

    mcic = pitchtools.NumberedIntervalClass.from_pitch_carriers(
        NamedPitch(12), NamedPitch(9.5))
    assert mcic == pitchtools.NumberedIntervalClass(-2.5)



# -*- coding: utf-8 -*-
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadObject


class Block(AbjadObject):
    '''A LilyPond file block.

    ..  container:: example

        ::

            >>> block = lilypondfiletools.Block(name='paper')
            >>> block.left_margin = lilypondfiletools.LilyPondDimension(2, 'cm')
            >>> block.right_margin = lilypondfiletools.LilyPondDimension(2, 'cm')
            >>> block
            <Block(name='paper')>

        ::

            >>> print(format(block))
            \paper {
                left-margin = 2\cm
                right-margin = 2\cm
            }

    ..  container:: example

        ::

            >>> block = lilypondfiletools.Block(name='score')
            >>> markup = Markup('foo')
            >>> block.items.append(markup)
            >>> block
            <Block(name='score')>

        ::

            >>> print(format(block))
            \score {
                {
                    \markup { foo }
                }
            }

    '''

    ### INITIALIZER ###

    def __init__(self, name='score'):
        assert isinstance(name, str), repr(name)
        self._name = name
        escaped_name = r'\{}'.format(name)
        self._escaped_name = escaped_name
        self._items = []

    ### SPECIAL METHODS ###

    def __format__(self, format_specification=''):
        r'''Formats block.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'lilypond'):
            return self._get_lilypond_format()
        elif format_specification == 'storage':
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __getitem__(self, name):
        r'''Gets item with `name`.

        ..  container:: example

            Gets score with name ``'Red Example Score'`` in score block:

            ::

                >>> block = lilypondfiletools.Block(name='score')
                >>> score = Score(name='Red Example Score')
                >>> block.items.append(score)

            ::

                >>> block['Red Example Score']
                Score(is_simultaneous=True)

        Returns item.

        Raises key error when no item with `name` is found.
        '''
        for item in self.items:
            if getattr(item, 'name', None) == name:
                return item
        raise KeyError

    ### PRIVATE PROPERTIES ###

    @property
    def _formatted_context_blocks(self):
        from abjad.tools import lilypondfiletools
        result = []
        context_blocks = []
        for item in self.items:
            if isinstance(item, lilypondfiletools.ContextBlock):
                context_blocks.append(item)
        for context_block in context_blocks:
            result.extend(context_block._get_format_pieces())
        return result

    ### PRIVATE METHODS ###

    def _format_item(self, item, depth=1):
        from abjad.tools import systemtools
        indent = systemtools.LilyPondFormatManager.indent * depth
        result = []
        if isinstance(item, (list, tuple)):
            result.append(indent + '{')
            for x in item:
                result.extend(self._format_item(x, depth + 1))
            result.append(indent + '}')
        elif isinstance(item, str):
            string = indent + item
            result.append(string)
        elif '_get_format_pieces' in dir(item):
            pieces = item._get_format_pieces()
            pieces = (indent + item for item in pieces)
            result.extend(pieces)
        return result

    def _get_format_pieces(self):
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        from abjad.tools import scoretools
        from abjad.tools import systemtools
        indent = systemtools.LilyPondFormatManager.indent
        result = []
        if (not self._get_formatted_user_attributes() and
            not getattr(self, 'contexts', None) and
            not getattr(self, 'context_blocks', None) and
            not len(self.items)):
            if self.name == 'score':
                return ''
            string = '{} {{}}'.format(self._escaped_name)
            result.append(string)
            return result
        string = '{} {{'.format(self._escaped_name)
        result.append(string)
        prototype = (scoretools.Leaf, markuptools.Markup)
        for item in self.items:
            if isinstance(item, lilypondfiletools.ContextBlock):
                continue
            if isinstance(item, prototype):
                item = [item]
            result.extend(self._format_item(item))
        formatted_attributes = self._get_formatted_user_attributes()
        formatted_attributes = [indent + x for x in formatted_attributes]
        result.extend(formatted_attributes)
        formatted_context_blocks = getattr(
            self, '_formatted_context_blocks', [])
        formatted_context_blocks = [
            indent + x for x in formatted_context_blocks]
        result.extend(formatted_context_blocks)
        result.append('}')
        return result

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            repr_is_bracketed=True,
            repr_is_indented=False,
            )

    def _get_formatted_user_attributes(self):
        from abjad.tools import indicatortools
        from abjad.tools import lilypondfiletools
        from abjad.tools import markuptools
        from abjad.tools import schemetools
        result = []
        for value in self.items:
            acceptable_types = (
                schemetools.Scheme,
                indicatortools.LilyPondCommand,
                )
            if isinstance(value, acceptable_types):
                result.append(format(value, 'lilypond'))
        for key, value in sorted(vars(self).items()):
            if not key.startswith('_'):
                # format subkeys via double underscore
                formatted_key = key.split('__')
                for i, k in enumerate(formatted_key):
                    formatted_key[i] = k.replace('_', '-')
                    if 0 < i:
                        string = "#'{}".format(formatted_key[i])
                        formatted_key[i] = string
                formatted_key = ' '.join(formatted_key)
                # format value
                accetable_types = (
                    schemetools.Scheme,
                    lilypondfiletools.LilyPondDimension,
                    indicatortools.LilyPondCommand,
                    )
                if isinstance(value, markuptools.Markup):
                    formatted_value = value._get_format_pieces()
                elif isinstance(value, accetable_types):
                    formatted_value = [format(value, 'lilypond')]
                else:
                    formatted_value = schemetools.Scheme(value)
                    formatted_value = format(formatted_value, 'lilypond')
                    formatted_value = [formatted_value]
                setting = '{!s} = {!s}'
                setting = setting.format(
                    formatted_key,
                    formatted_value[0],
                    )
                result.append(setting)
                result.extend(formatted_value[1:])
        return result

    def _get_lilypond_format(self):
        return '\n'.join(self._get_format_pieces())

    ### PUBLIC PROPERTIES ###

    @property
    def items(self):
        r'''Gets items in block.

        ..  container:: example

            ::

                >>> block = lilypondfiletools.Block(name='score')
                >>> markup = Markup('foo')
                >>> block.items.append(markup)

            ::

                >>> block.items
                [Markup(contents=['foo'])]

        Returns list.
        '''
        return self._items

    @property
    def name(self):
        r'''Gets name of block.

        ..  container:: example

            ::

                >>> block = lilypondfiletools.Block(name='score')
                >>> markup = Markup('foo')
                >>> block.items.append(markup)

            ::

                >>> block.name
                'score'

        Returns string.
        '''
        return self._name



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_MutationAgent_splice_01():

    voice = Voice("c'8 d'8 e'8")
    beam = Beam()
    attach(beam, voice[:])

    result = mutate(voice[-1]).splice(
        [Note("c'8"), Note("d'8"), Note("e'8")],
        grow_spanners=True,
        )

    assert inspect_(voice).is_well_formed()
    assert result == voice[-4:]
    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8
            c'8
            d'8
            e'8 ]
        }
        '''
        )


def test_agenttools_MutationAgent_splice_02():
    r'''Splices leaf after interior leaf.
    '''

    voice = Voice("c'8 d'8 e'8")
    beam = Beam()
    attach(beam, voice[:])
    result = mutate(voice[1]).splice(
        [Note("dqs'8")],
        grow_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            dqs'8
            e'8 ]
        }
        '''
        )

    assert result == voice[1:3]
    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_splice_03():
    r'''Splices tuplet after tuplet.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    voice = Voice([tuplet])
    beam = Beam()
    attach(beam, tuplet[:])
    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    result = mutate(voice[-1]).splice(
        [tuplet],
        grow_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8 [
                d'8
                e'8
            }
            \times 2/3 {
                c'8
                d'8
                e'8 ]
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert result == voice[:]


def test_agenttools_MutationAgent_splice_04():
    r'''Splices after container with underspanners.
    '''

    voice = Voice(Container("c'8 c'8") * 2)
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    result = mutate(voice[0]).splice(
        [Note("dqs'8")],
        grow_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [
                c'8
            }
            dqs'8
            {
                c'8
                c'8 ]
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert result == voice[0:2]


def test_agenttools_MutationAgent_splice_05():
    r'''Extends leaves rightwards after leaf.
    '''

    voice = Voice("c'8 d'8 e'8")
    beam = Beam()
    attach(beam, voice[:])

    result = mutate(voice[-1]).splice(
        [Note("c'8"), Note("d'8"), Note("e'8")],
        grow_spanners=False,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            e'8 ]
            c'8
            d'8
            e'8
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert result == voice[-4:]


def test_agenttools_MutationAgent_splice_06():
    r'''Extends leaf rightwards after interior leaf.
    '''

    voice = Voice("c'8 d'8 e'8")
    beam = Beam()
    attach(beam, voice[:])

    result = mutate(voice[1]).splice(
        [Note("dqs'8")],
        grow_spanners=False,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            d'8
            dqs'8
            e'8 ]
        }
        '''
        )

    assert result == voice[1:3]
    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_splice_07():
    r'''Splices leaves left of leaf.
    '''

    voice = Voice("c'8 d'8 e'8")
    beam = Beam()
    attach(beam, voice[:])
    notes = [Note("c'16"), Note("d'16"), Note("e'16")]
    result = mutate(voice[0]).splice(
        notes,
        direction=Left,
        grow_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'16 [
            d'16
            e'16
            c'8
            d'8
            e'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert result == voice[:4]


def test_agenttools_MutationAgent_splice_08():
    r'''Splices leaf left of interior leaf.
    '''

    voice = Voice("c'8 d'8 e'8")
    beam = Beam()
    attach(beam, voice[:])
    result = mutate(voice[1]).splice(
        [Note("dqf'8")],
        direction=Left,
        grow_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            dqf'8
            d'8
            e'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert result == voice[1:3]


def test_agenttools_MutationAgent_splice_09():
    r'''Splices tuplet left of tuplet.
    '''

    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    voice = Voice([tuplet])
    beam = Beam()
    attach(beam, tuplet[:])
    tuplet = scoretools.FixedDurationTuplet(Duration(2, 8), "c'8 d'8 e'8")
    result = mutate(voice[0]).splice(
        [tuplet],
        direction=Left,
        grow_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            \times 2/3 {
                c'8 [
                d'8
                e'8
            }
            \times 2/3 {
                c'8
                d'8
                e'8 ]
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert result == voice[:]


def test_agenttools_MutationAgent_splice_10():
    r'''Splices left of container with underspanners.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)
    result = mutate(voice[1]).splice(
        [Note("dqs'8")],
        direction=Left,
        grow_spanners=True,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8 [
                d'8
            }
            dqs'8
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    assert result == voice[1:]
    assert inspect_(voice).is_well_formed()


def test_agenttools_MutationAgent_splice_11():
    r'''Extends leaves leftwards of leaf. Do not extend edge spanners.
    '''

    voice = Voice("c'8 d'8 e'8")
    beam = Beam()
    attach(beam, voice[:])
    notes = [Note("c'16"), Note("d'16"), Note("e'16")]
    result = mutate(voice[0]).splice(
        notes,
        direction=Left,
        grow_spanners=False,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'16
            d'16
            e'16
            c'8 [
            d'8
            e'8 ]
        }
        '''
        )

    assert inspect_(voice).is_well_formed()
    assert result == voice[:4]


def test_agenttools_MutationAgent_splice_12():
    r'''Extends leaf leftwards of interior leaf. Does extend interior spanners.
    '''

    voice = Voice("c'8 d'8 e'8")
    beam = Beam()
    attach(beam, voice[:])
    result = mutate(voice[1]).splice(
        [Note("dqf'8")],
        direction=Left,
        grow_spanners=False,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            dqf'8
            d'8
            e'8 ]
        }
        '''
        )

    assert result == voice[1:3]
    assert inspect_(voice).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_MultipartBeam___format___01():

    container = Container("c'8 d'8 r8 e'8 f'8 g'4")
    beam = spannertools.MultipartBeam()
    attach(beam, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8 [
            d'8 ]
            r8
            e'8 [
            f'8 ]
            g'4
        }
        '''
        )


def test_spannertools_MultipartBeam___format___02():

    container = Container("c'8 r4 c'8")
    beam = spannertools.MultipartBeam()
    attach(beam, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            r4
            c'8
        }
        '''
        )


def test_spannertools_MultipartBeam___format___03():

    container = Container("c'8. r16 c'8. r16")
    beam = spannertools.MultipartBeam()
    attach(beam, container[:])

    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8.
            r16
            c'8.
            r16
        }
        '''
        )


# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_GraceContainer_01():
    r'''Grace music is a container.
    '''

    gracecontainer = scoretools.GraceContainer(
        [Note(0, (1, 16)), Note(2, (1, 16)), Note(4, (1, 16))])

    assert format(gracecontainer) == stringtools.normalize(
        r'''
        \grace {
            c'16
            d'16
            e'16
        }
        '''
        )

    assert isinstance(gracecontainer, Container)
    assert len(gracecontainer) == 3


def test_scoretools_GraceContainer_02():
    r'''GraceContainer.kind is managed attribute.
    GraceContainer.kind knows about "after", "grace",
    "acciaccatura", "appoggiatura".
    '''

    gracecontainer = scoretools.GraceContainer(
        [Note(0, (1, 16)), Note(2, (1, 16)), Note(4, (1, 16))])
    gracecontainer.kind = 'acciaccatura'
    assert gracecontainer.kind == 'acciaccatura'
    gracecontainer.kind = 'grace'
    assert gracecontainer.kind == 'grace'
    gracecontainer.kind = 'after'
    assert gracecontainer.kind == 'after'
    gracecontainer.kind = 'appoggiatura'
    assert gracecontainer.kind == 'appoggiatura'
    assert pytest.raises(Exception, 'gracecontainer.kind = "blah"')


def test_scoretools_GraceContainer_03():
    r'''Grace formats correctly as grace.
    '''

    gracecontainer = scoretools.GraceContainer("c'8 c'8 c'8")
    gracecontainer.kind = 'grace'
    assert format(gracecontainer) == stringtools.normalize(
        r'''
        \grace {
            c'8
            c'8
            c'8
        }
        '''
        )


def test_scoretools_GraceContainer_04():
    r'''Grace formats correctly as acciaccatura.
    '''

    gracecontainer = scoretools.GraceContainer("c'8 c'8 c'8")
    gracecontainer.kind = 'acciaccatura'
    assert format(gracecontainer) == stringtools.normalize(
        r'''
        \acciaccatura {
            c'8
            c'8
            c'8
        }
        '''
        )


def test_scoretools_GraceContainer_05():
    r'''Grace formats correctly as appoggiatura.
    '''

    gracecontainer = scoretools.GraceContainer("c'8 c'8 c'8")
    gracecontainer.kind = 'appoggiatura'
    assert format(gracecontainer) == stringtools.normalize(
        r'''
        \appoggiatura {
            c'8
            c'8
            c'8
        }
        '''
        )


def test_scoretools_GraceContainer_06():
    r'''Grace formats correctly as after grace.
    '''

    gracecontainer = scoretools.GraceContainer("c'8 c'8 c'8")
    gracecontainer.kind = 'after'
    assert format(gracecontainer) == stringtools.normalize(
        r'''
        {
            c'8
            c'8
            c'8
        }
        '''
        )


def test_scoretools_GraceContainer_07():
    r'''Grace containers can be appended.
    '''

    gracecontainer = scoretools.GraceContainer("c'8 c'8")
    note = Note(1, (1, 4))
    gracecontainer.append(note)
    assert len(gracecontainer) == 3
    assert gracecontainer[-1] is note


def test_scoretools_GraceContainer_08():
    r'''Grace containers can be extended.
    '''

    gracecontainer = scoretools.GraceContainer("c'8 c'8")
    ns = Note(1, (1, 4)) * 2
    gracecontainer.extend(ns)
    assert len(gracecontainer) == 4
    assert tuple(gracecontainer[-2:]) == tuple(ns)


# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Staff___setattr___01():
    r'''Slots constrain staff attributes.
    '''

    staff = Staff([])

    assert pytest.raises(AttributeError, "staff.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_tonalanalysistools_RootedChordClass_extent_to_cardinality_01():

    assert tonalanalysistools.RootedChordClass.extent_to_cardinality(5) == 3
    assert tonalanalysistools.RootedChordClass.extent_to_cardinality(7) == 4
    assert tonalanalysistools.RootedChordClass.extent_to_cardinality(9) == 5
    assert tonalanalysistools.RootedChordClass.extent_to_cardinality(11) == 6
    assert tonalanalysistools.RootedChordClass.extent_to_cardinality(13) == 7


def test_tonalanalysistools_RootedChordClass_extent_to_cardinality_02():

    string = 'tonalanalysistools.RootedChordClass.extent_to_cardinality(1)'
    assert pytest.raises(Exception, string)



# -*- coding: utf-8 -*-
import copy
from abjad.tools import indicatortools
from abjad.tools import markuptools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate


def apply_bowing_marks(score):
    r'''Applies bowing marks to score.
    '''

    # apply alternating upbow and downbow for first two sounding bars
    # of the first violin
    for measure in score['First Violin Voice'][6:8]:
        for i, chord in enumerate(iterate(measure).by_class(scoretools.Chord)):
            if i % 2 == 0:
                articulation = indicatortools.Articulation('downbow')
                attach(articulation, chord)
            else:
                articulation = indicatortools.Articulation('upbow')
                attach(articulation, chord)

    # create and apply rebowing markup
    rebow_markup = markuptools.Markup.concat([
        markuptools.Markup.musicglyph('scripts.downbow'),
        markuptools.Markup.hspace(1),
        markuptools.Markup.musicglyph('scripts.upbow'),
        ])
    markup = copy.copy(rebow_markup)
    attach(markup, score['First Violin Voice'][64][0])
    markup = copy.copy(rebow_markup)
    attach(markup, score['Second Violin Voice'][75][0])
    markup = copy.copy(rebow_markup)
    attach(markup, score['Viola Voice'][86][0])


# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Container_pop_01():
    r'''Containers pop leaves correctly.
    Popped leaves detach from parent.
    Popped leaves withdraw from crossing spanners.
    Popped leaves carry covered spanners forward.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    slur = Slur()
    attach(slur, voice[:])
    beam = Beam()
    attach(beam, voice[1])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 (
            d'8 [ ]
            e'8
            f'8 )
        }
        '''
        )

    result = voice.pop(1)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 (
            e'8
            f'8 )
        }
        '''
        )

    assert inspect_(voice).is_well_formed()

    "Result is now d'8 [ ]"

    assert inspect_(result).is_well_formed()
    assert format(result) == "d'8 [ ]"


def test_scoretools_Container_pop_02():
    r'''Containers pop nested containers correctly.
    Popped containers detach from both parent and spanners.
    '''

    staff = Staff("{ c'8 d'8 } { e'8 f'8 }")
    leaves = select(staff).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                d'8
            }
            {
                e'8
                f'8 ]
            }
        }
        '''
        )

    sequential = staff.pop()

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                c'8 [
                d'8 ]
            }
        }
        '''
        )
    assert inspect_(staff).is_well_formed()

    assert format(sequential) == stringtools.normalize(
        r'''
        {
            e'8
            f'8
        }
        '''
        )

    assert inspect_(sequential).is_well_formed()
